# -*- coding: utf-8 -*-
# 国信托管接口
import os.path
import hashlib
import unittest
import datetime as dt
import json
import time
import os
import copy
import logging
import numpy as np
import itertools
import pandas as pd
from retrying import retry
from dateutil.relativedelta import relativedelta
from dateutil import rrule, parser
from enum import Enum
from urllib import parse
from rio3.log.setup import setup_file_screen_logger
from rio3.utils.http_requests import BaseRequests

# 初始化log_handler
logger = logging.getLogger(__name__)
setup_file_screen_logger(name=__name__)

SYSTEM_AUTH_DICT = {
    'dashboard': {
        'HOST': 'http://dashboard.xuanyeinvest.com',
        'PORT': 8001,
    },
    'dashboard_dev': {
        'HOST': 'http://localhost',
        'PORT': 8001,
    },
    'operation': {
        'HOST': 'https://operation.xuanyeinvest.com',
        'PORT': 9001
    }
}


class SystemApi(BaseRequests):
    def __init__(self, system_name='dashboard', options=None):
        if options is not None:
            self.options = options
        else:
            options = SYSTEM_AUTH_DICT.get(system_name, None)
            self.options = SYSTEM_AUTH_DICT['dashboard'] if options is None else options
        self.base_url = ':'.join([self.options['HOST'], str(self.options['PORT'])])

    def get_url(self, request_url):
        return parse.urljoin(self.base_url, request_url)

    def get_params(self, param_dict):
        param_dict_copy = copy.copy(param_dict)
        return param_dict_copy

    def post_request(self, request_url, data={}):
        status_code, response = self.post(url=self.get_url(request_url=request_url), data=data, timeout=60)
        return status_code, response


class TestCaseIO(unittest.TestCase):
    def setUp(self):
        self.logger = logger

    def tearDown(self):
        pass

    def test_get_dashboard_trading_info(self):
        system_api = SystemApi(system_name='dashboard')
        request_url = 'trade/get_trading_info_data'
        param_dict = {
            'account_type': 'position',
            'manual_refresh': 'true'
        }
        code, result = system_api.post_request(request_url=request_url, data=param_dict)
        pass

    def test_get_netvalue_data(self):
        system_api = SystemApi(system_name='dashboard_dev')
        request_url = 'netvalue/get_trading_account_data_request'
        param_dict = {
            'quote_type': 'future',  # 指定访问经纪类别 future/stock/option
            'investor_id[]': [],  # 账号列表  空数组代表获取所有 []
            'start_date': '2022-03-01',
            'end_date': '2022-04-05',
            'filter_dw': 0,  # 过滤出入金 单位是万
            'filter_balance': 1,  # 过滤总资金 单位是万
            'is_account_detail': 'on',  # 是否显示账号详细
            'with_summary': 'off',  # 是否显示统计行
            'account_type[]': ['POSITION']  # 账户类型  CTA/MOM/POSITION 空数组代表所有 []
        }
        code, result = system_api.post_request(request_url=request_url, data=param_dict)
        result_df = pd.DataFrame.from_dict(result['table_data'])
        pass


if __name__ == '__main__':
    unittest.main()
