# -*- coding:utf-8 -*-
import logging
import os
import datetime as dt
import unittest


class BaseObject(object):
    def __init__(self, name):
        self.name = name
        self.logger = logging.getLogger('root')


class Singleton(object):
    # 抽象单例
    def __new__(cls, *args, **kw):
        if not hasattr(cls, '_instance'):
            orig = super(Singleton, cls)
            cls._instance = orig.__new__(cls, *args, **kw)
        return cls._instance


class mytest(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_something(self):
        self.assertEqual(True, False)


def setup_logger(logger_dir='.', name="root", ch_level=logging.INFO, file_level=logging.DEBUG):
    if not os.path.exists(logger_dir):
        os.mkdir(logger_dir)
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)

    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

    ch = logging.StreamHandler()
    ch.setLevel(ch_level)
    ch.setFormatter(formatter)

    log_file = logger_dir + os.sep + os.path.basename(os.path.abspath(__file__)) + ".log"

    fh = logging.FileHandler(log_file, "w")
    fh.setLevel(file_level)
    fh.setFormatter(formatter)

    logger.addHandler(fh)
    logger.addHandler(ch)

    return logger, fh


if __name__ == "__main__":
    logger_dir = os.path.dirname(os.path.abspath(__file__)) + os.sep + "log" + dt.datetime.today().strftime("%m%d")
    logger, file_handler = setup_logger(logger_dir=logger_dir, name="root", ch_level=logging.INFO,
                                        file_level=logging.DEBUG)
    unittest.main()
