# coding: utf-8
from sqlalchemy import Column, Date, DateTime, Float, ForeignKey, Index, String, Table, text
from sqlalchemy.dialects.mysql import BIGINT, BIT, DATETIME, INTEGER, LONGTEXT, SMALLINT, TINYINT
from sqlalchemy.orm import relationship
from sqlalchemy.ext.declarative import declarative_base

Base = declarative_base()
metadata = Base.metadata

t_vw_m2m_account2action = Table(
    'vw_m2m_account2action', metadata,
    Column('id', BIGINT(20), server_default=text("'0'")),
    Column('item_name', String(255)),
    Column('item_description', String(500)),
    Column('investor_id', String(255)),
    Column('account_num', String(50)),
    Column('memo', String(500)),
    Column('security_company_name', String(100)),
    Column('product_name', String(255)),
    Column('security_type', INTEGER(3)),
    Column('category_name', String(255)),
    Column('category_description', String(500)),
    Column('id_action_item', BIGINT(20)),
    Column('id_operation_action_category', INTEGER(11)),
    Column('id_product_info', BIGINT(20)),
    Column('id_security_company', INTEGER(11)),
    Column('id_product_sync', BIGINT(20)),
    Column('id_product_account', BIGINT(20), server_default=text("'0'")),
    Column('update_time', DateTime)
)

t_vw_operation_account2upload = Table(
    'vw_operation_account2upload', metadata,
    Column('id', BIGINT(20), server_default=text("'0'")),
    Column('product_name', String(255)),
    Column('product_code', String(255)),
    Column('org_file_name', String(255)),
    Column('store_file_name', String(255)),
    Column('category_name', String(255)),
    Column('security_type', INTEGER(3)),
    Column('investor_id', String(255)),
    Column('account_num', String(50)),
    Column('id_product_account', BIGINT(20)),
    Column('id_upload', BIGINT(20), server_default=text("'0'")),
    Column('id_operation_action_category', INTEGER(11)),
    Column('update_time', DateTime)
)

t_vw_operation_account_balance_info = Table(
    'vw_operation_account_balance_info', metadata,
    Column('id', BIGINT(20), server_default=text("'0'")),
    Column('account_type', String(255)),
    Column('account_num', String(50)),
    Column('security_company_name', String(255)),
    Column('custody_name', String(255)),
    Column('custody_code', String(100)),
    Column('product_name', String(255)),
    Column('product_code', String(255)),
    Column('investor_id', String(255)),
    Column('date_trading', Date),
    Column('balance', Float(asdecimal=True)),
    Column('available', Float(asdecimal=True)),
    Column('margin', Float(asdecimal=True)),
    Column('deposit', Float(asdecimal=True)),
    Column('withdraw', Float(asdecimal=True)),
    Column('settlement_dw', Float(asdecimal=True)),
    Column('profit', Float(asdecimal=True)),
    Column('memo', String(255)),
    Column('commission', Float(asdecimal=True)),
    Column('fee', Float(asdecimal=True)),
    Column('margin_account', String(100)),
    Column('margin_password', String(100)),
    Column('date_update', DateTime),
    Column('id_security_company', INTEGER(11)),
    Column('id_product_info', BIGINT(20)),
    Column('id_account_type', INTEGER(11)),
    Column('id_custody', INTEGER(11))
)

t_vw_operation_account_balance_info_full = Table(
    'vw_operation_account_balance_info_full', metadata,
    Column('id', BIGINT(20), server_default=text("'0'")),
    Column('account_type', String(255)),
    Column('account_num', String(50)),
    Column('security_company_name', String(255)),
    Column('custody_name', String(255)),
    Column('custody_code', String(100)),
    Column('product_name', String(255)),
    Column('product_code', String(255)),
    Column('investor_id', String(255)),
    Column('date_trading', Date),
    Column('balance', Float(asdecimal=True)),
    Column('available', Float(asdecimal=True)),
    Column('margin', Float(asdecimal=True)),
    Column('deposit', Float(asdecimal=True)),
    Column('withdraw', Float(asdecimal=True)),
    Column('settlement_dw', Float(asdecimal=True)),
    Column('profit', Float(asdecimal=True)),
    Column('memo', String(255)),
    Column('commission', Float(asdecimal=True)),
    Column('fee', Float(asdecimal=True)),
    Column('margin_account', String(100)),
    Column('margin_password', String(100)),
    Column('date_update', DateTime),
    Column('id_security_company', INTEGER(11)),
    Column('id_product_info', BIGINT(20)),
    Column('id_account_type', INTEGER(11)),
    Column('id_custody', INTEGER(11))
)

t_vw_operation_accountinfo_sync = Table(
    'vw_operation_accountinfo_sync', metadata,
    Column('id', BIGINT(20), server_default=text("'0'")),
    Column('investor_id', String(255)),
    Column('account_num', String(50)),
    Column('account_name', String(500)),
    Column('flg_valid', BIT(1)),
    Column('register_date', Date),
    Column('account_type', String(255)),
    Column('account_category', String(255)),
    Column('account_category_code', String(100)),
    Column('id_account_type', INTEGER(11)),
    Column('id_account_category', INTEGER(11)),
    Column('create_date', DateTime),
    Column('update_date', DateTime)
)

t_vw_operation_action_item = Table(
    'vw_operation_action_item', metadata,
    Column('id', BIGINT(20), server_default=text("'0'")),
    Column('item_name', String(255)),
    Column('item_description', String(500)),
    Column('category_name', String(255)),
    Column('category_description', String(500)),
    Column('security_type', INTEGER(3)),
    Column('id_operation_action_category', INTEGER(11)),
    Column('update_time', DateTime)
)

t_vw_operation_custody_netvalue_sync = Table(
    'vw_operation_custody_netvalue_sync', metadata,
    Column('id', BIGINT(20), server_default=text("'0'")),
    Column('product_code', String(255)),
    Column('product_name', String(300)),
    Column('fund_type', String(10)),
    Column('custody_name', String(255)),
    Column('custody_code', String(100)),
    Column('nav_date', Date),
    Column('asset_vol', Float(asdecimal=True)),
    Column('asset_nav', Float(asdecimal=True)),
    Column('total_asset', Float(asdecimal=True)),
    Column('nav', Float(asdecimal=True)),
    Column('accumulative_nav', Float(asdecimal=True)),
    Column('remark1', String(500)),
    Column('remark2', String(500)),
    Column('id_custody', INTEGER(11)),
    Column('create_date', DateTime),
    Column('update_date', DateTime)
)

t_vw_operation_custody_share_sync = Table(
    'vw_operation_custody_share_sync', metadata,
    Column('id', BIGINT(20), server_default=text("'0'")),
    Column('id_custody', INTEGER(11)),
    Column('customer_name', String(255)),
    Column('customer_type', String(30)),
    Column('custody_code', String(100)),
    Column('custody_name', String(255)),
    Column('certificate_type', String(50)),
    Column('certificate_no', String(30)),
    Column('ta_account_id', String(20)),
    Column('transaction_account_id', String(30)),
    Column('product_name', String(300)),
    Column('product_code', String(255)),
    Column('vol_date', Date),
    Column('total_fund_vol', Float(asdecimal=True)),
    Column('application_vol', Float(asdecimal=True)),
    Column('frozen_vol', Float(asdecimal=True)),
    Column('dividend_method', String(50)),
    Column('distributor_code', String(50)),
    Column('distributor_name', String(300)),
    Column('remark1', String(500)),
    Column('remark2', String(500)),
    Column('create_date', DateTime),
    Column('update_date', DateTime),
    Column('query_date', Date)
)

t_vw_operation_grouped_accountinfo_sync = Table(
    'vw_operation_grouped_accountinfo_sync', metadata,
    Column('id', BIGINT(20), server_default=text("'0'")),
    Column('account_type', String(255)),
    Column('investor_id', String(255)),
    Column('account_num', String(50)),
    Column('account_name', String(500)),
    Column('group_name', String(255)),
    Column('group_code', String(50)),
    Column('account_category', String(255)),
    Column('account_category_code', String(100)),
    Column('flg_valid', BIT(1)),
    Column('register_date', Date),
    Column('id_accountinfo_sync', BIGINT(20)),
    Column('id_accountinfo_sync_group', INTEGER(11)),
    Column('id_account_category', INTEGER(11)),
    Column('id_account_type', INTEGER(11))
)

t_vw_operation_product2upload = Table(
    'vw_operation_product2upload', metadata,
    Column('id', BIGINT(20), server_default=text("'0'")),
    Column('product_name', String(255)),
    Column('org_file_name', String(255)),
    Column('store_file_name', String(255)),
    Column('category_name', String(255)),
    Column('security_type', INTEGER(3)),
    Column('id_product_info', BIGINT(20)),
    Column('id_upload', BIGINT(20)),
    Column('id_operation_action_category', INTEGER(11)),
    Column('update_time', DateTime)
)

t_vw_operation_product_account = Table(
    'vw_operation_product_account', metadata,
    Column('id', BIGINT(20), server_default=text("'0'")),
    Column('product_name', String(255)),
    Column('product_code', String(255)),
    Column('security_company_name', String(255)),
    Column('security_company_code', String(100)),
    Column('security_type', INTEGER(3)),
    Column('investor_id', String(255)),
    Column('account_num', String(50)),
    Column('margin_account', String(100)),
    Column('margin_password', String(100)),
    Column('update_time', DateTime),
    Column('memo', String(800)),
    Column('id_product_info', BIGINT(20), server_default=text("'0'")),
    Column('id_product_sync', BIGINT(20)),
    Column('id_security_company', INTEGER(11))
)

t_vw_operation_product_customer = Table(
    'vw_operation_product_customer', metadata,
    Column('id', BIGINT(20), server_default=text("'0'")),
    Column('product_name', String(255)),
    Column('product_code', String(255)),
    Column('customer_name', String(255)),
    Column('customer_status', INTEGER(11)),
    Column('custody_code', String(100)),
    Column('custody_name', String(255)),
    Column('unit_share', Float(asdecimal=True)),
    Column('unit_share_date', Date),
    Column('customer_type', String(255)),
    Column('memo', LONGTEXT),
    Column('sales_org_name', String(255)),
    Column('sales_memo', String(800)),
    Column('update_time', DateTime),
    Column('id_product_info', BIGINT(20)),
    Column('id_customer_sync', BIGINT(20)),
    Column('id_sales_organization', INTEGER(11)),
    Column('id_customer_type', INTEGER(11)),
    Column('id_custody', INTEGER(11))
)

t_vw_operation_product_info = Table(
    'vw_operation_product_info', metadata,
    Column('id', BIGINT(20), server_default=text("'0'")),
    Column('name', String(255)),
    Column('product_code', String(255)),
    Column('fund_share_type', String(100)),
    Column('open_date', String(255)),
    Column('settlement_date', String(255)),
    Column('custody_name', String(255)),
    Column('date_found', DateTime),
    Column('custody_fee', Float(asdecimal=True)),
    Column('management_fee', Float(asdecimal=True)),
    Column('profit_reward', String(255)),
    Column('precaution', Float(asdecimal=True)),
    Column('stop_loss', Float(asdecimal=True)),
    Column('product_manager', String(255)),
    Column('product_type', INTEGER(11)),
    Column('scope', String(800)),
    Column('custody_code', String(100)),
    Column('memo', String(500)),
    Column('id_custody', INTEGER(11)),
    Column('id_product_sync', BIGINT(20)),
    Column('product_property_id', BIGINT(20)),
    Column('update_time', DateTime)
)

t_vw_operation_product_info_fee = Table(
    'vw_operation_product_info_fee', metadata,
    Column('id', BIGINT(20), server_default=text("'0'")),
    Column('product_code', String(255)),
    Column('product_name', String(255)),
    Column('fee_type', String(255)),
    Column('fee_type_code', String(100)),
    Column('val_date', Date),
    Column('year', INTEGER(4)),
    Column('quarter', INTEGER(1)),
    Column('fee', Float(asdecimal=True)),
    Column('memo', String(500)),
    Column('custody_fee', Float(asdecimal=True)),
    Column('management_fee', Float(asdecimal=True)),
    Column('profit_reward', String(255)),
    Column('precaution', Float(asdecimal=True)),
    Column('stop_loss', Float(asdecimal=True)),
    Column('id_product_info', BIGINT(20)),
    Column('id_fee_type', INTEGER(11)),
    Column('id_product_sync', BIGINT(20)),
    Column('create_date', DateTime),
    Column('update_date', DateTime)
)

t_vw_operation_risk_management = Table(
    'vw_operation_risk_management', metadata,
    Column('id', BIGINT(20), server_default=text("'0'")),
    Column('account_num', String(50)),
    Column('investor_id', String(255)),
    Column('account_name', String(500)),
    Column('account_category', String(255)),
    Column('account_category_code', String(100)),
    Column('group_name', String(255)),
    Column('group_code', String(50)),
    Column('category_source', String(50)),
    Column('start_date', Date),
    Column('settlement_date', String(255)),
    Column('precaution', Float(asdecimal=True)),
    Column('stop_loss', Float(asdecimal=True)),
    Column('charge_mode', String(255)),
    Column('initial_balance', Float(asdecimal=True)),
    Column('memo', String(500)),
    Column('create_date', DateTime),
    Column('update_date', DateTime),
    Column('id_account_category', INTEGER(11)),
    Column('id_accountinfo_sync', BIGINT(20)),
    Column('id_accountinfo_group', INTEGER(11))
)
