# -*- coding: utf-8 -*-
import os
import traceback
import datetime as dt
import pandas as pd
import unittest
import numpy as np
import logging
import pyodbc
from dateutil.relativedelta import relativedelta
from rio3.base import constant
from rio3.operation import db_operation_models as db_oper_models
from rio3.operation import db_web_models
from sqlalchemy import create_engine, or_, and_, desc, asc, sql
from sqlalchemy.sql import func
from rio3.datafeed.dbIO import get_db_session, get_previous_tday
from rio3.trade.future.feed import get_trading_account_data

logger = logging.getLogger('root')


# 宣夜信熹季添利一期私募证券投资基金
# 宣夜投资涓流定制一号私募证券投资基金
# 宣夜投资季添利一期私募证券投资基金
# 宣夜投资稳澜一号私募证券投资基金
# 宣夜投资CTA激流一号私募证券投资基金
# 宣夜飒露紫激流一号私募证券投资基金
# 宣夜益泽稳健配置二号私募证券投资基金
# 宣夜益泽稳健配置一号私募证券投资基金
# 宣夜益泽大有一号私募证券投资基金
# 宣夜益泽一号私募证券投资基金
# 宣夜益泽500指数增强私募证券投资基金
# 宣夜投资谦盈二期私募证券投资基金
# 宣夜投资谦盈三期私募证券投资基金
# 宣夜投资谦盈一期私募证券投资基金
# 宣夜投资至信稳健号私募证券投资基金
# 宣夜投资稳赢三号私募证券投资基金
# 宣夜投资涓流二号私募证券投资基金
# 宣夜投资涓流一号私募证券投资基金
# 宣夜投资泰盈二期私募证券投资基金
# 宣夜投资泰盈一期私募证券投资基金
# 宣夜投资期权稳健成长一期私募证券投资基金
# 宣夜投资昀泽一号私募证券投资基金
# 宣夜投资惠盈一号私募证券投资基金
# 宣夜投资引路者一号私募证券投资基金
# 宣夜投资市场中性禄有一号私募证券投资基金
# 宣夜投资市场中性源流一号私募证券投资基金
# 宣夜投资大有一号私募证券投资基金
# 宣夜投资启明一号私募证券投资基金
# 宣夜投资CTA稳流一期私募证券投资基金
# 宣夜投资CTA激流定制一号私募证券投资基金
# 宣夜投资500指数增强私募证券投资基金
# 宣夜投资500指数增强沧浪一号私募证券投资基金
# 宣夜南华尊享季添利1期私募证券投资基金
# 宣夜投资启明二号私募证券投资基金
# 宣夜投资启明三号私募证券投资基金
# 宣夜投资华鑫鑫诚私募证券投资基金
# 宣夜投资稳赢一号私募证券投资基金
# 宣夜投资稳赢二号私募证券投资基金
# 宣夜投资股指方舟一号私募证券投资基金


def get_list_parameter(para, para_type):
    list_para = [para, ] if isinstance(para, para_type) else para
    return list_para


# -------------------------------运营系统数据-----------------------------
def get_fund_investor_data(instance_id=None, fund_name=None, fund_code=None, investor_id=None, security_type_num=None,
                           db_string=None, dataframe=True):
    """
    获取基金和账号的关联关系
    :param instance_id: 实例id
    :param fund_name: 基金名称
    :param fund_code: 基金代码
    :param investor_id: 账户ID
    :param security_type_num: 账户经纪类别分类 0: 期货 1: 股票 2: 期权
    :return:
    """
    instance_id = get_list_parameter(para=instance_id, para_type=(str, int))
    fund_name = get_list_parameter(para=fund_name, para_type=str)
    fund_code = get_list_parameter(para=fund_code, para_type=str)
    investor_id = get_list_parameter(para=investor_id, para_type=str)
    security_type_num = get_list_parameter(para=security_type_num, para_type=int)
    db_string = constant.DB_OPERATION_RD_ENGINE_STRING if db_string is None else db_string
    try:
        session = get_db_session(db_string)
        q = session.query(db_oper_models.t_vw_operation_product_account)
        conditions = None
        if instance_id is not None:
            conditions = db_oper_models.t_vw_operation_product_account.columns['id'] == instance_id
        if fund_name is not None:
            if conditions is None:
                conditions = db_oper_models.t_vw_operation_product_account.columns['product_name'].in_(fund_name)
            else:
                conditions = and_(conditions,
                                  db_oper_models.t_vw_operation_product_account.columns['product_name'].in_(fund_name))
        if fund_code is not None:
            if conditions is None:
                conditions = db_oper_models.t_vw_operation_product_account.columns['product_code'].in_(fund_code)
            else:
                conditions = and_(conditions,
                                  db_oper_models.t_vw_operation_product_account.columns['product_code'].in_(fund_code))
        if investor_id is not None:
            if conditions is None:
                conditions = db_oper_models.t_vw_operation_product_account.columns['investor_id'].in_(investor_id)
            else:
                conditions = and_(conditions,
                                  db_oper_models.t_vw_operation_product_account.columns['investor_id'].in_(investor_id))
        if security_type_num is not None:
            if conditions is None:
                conditions = db_oper_models.t_vw_operation_product_account.columns['security_type'].in_(
                    security_type_num)
            else:
                conditions = and_(conditions,
                                  db_oper_models.t_vw_operation_product_account.columns['security_type'].in_(
                                      security_type_num))
        q = q.order_by(asc(db_oper_models.t_vw_operation_product_account.columns['product_name']))
        data_list = q.all() if conditions is None else q.filter(conditions).all()
        data_list = [{key: getattr(item, key) for key in item._fields} for item in data_list]
        if dataframe:
            data_df = pd.DataFrame.from_dict(data_list)
            return data_df
        else:
            return data_list
    except Exception as ex:
        print('读取产品账号关联数据失败:\r\n{}'.format(traceback.format_exc()))


def get_web_netvalue_data(instance_id=None, fund_name=None, fund_property=None, nav_start=constant.DATETIME_MIN,
                          nav_end=constant.DATETIME_MAX, db_string=None, is_latest=True):
    """
    获取官网净值数据
    :param instance_id:
    :param fund_name:
    :param fund_property:
    :param nav_start:
    :param nav_end:
    :param db_string:
    :param is_latest:
    :return:
    """
    list_instance_id = get_list_parameter(para=instance_id, para_type=(str, int))
    list_fund_name = get_list_parameter(para=fund_name, para_type=str)
    list_fund_property = get_list_parameter(para=fund_property, para_type=str)
    db_string = constant.DB_WEB_STRING if db_string is None else db_string
    # 默认获取最近一个月
    nav_end = dt.datetime.now().date() if nav_end is None else pd.to_datetime(nav_end).to_pydatetime()
    nav_start = constant.DATETIME_MIN if nav_start is None else pd.to_datetime(nav_start).to_pydatetime()
    nav_start_str = nav_start.strftime('%Y-%m-%d')
    nav_end_str = nav_end.strftime('%Y-%m-%d')
    try:
        conn = pyodbc.connect(db_string)
        if not is_latest:
            sql = f"""
            SELECT id, fund_name, fund_status, fund_property, nav_date, unit_nav, accumulative_nav from vw_fund_netvalue AS t1 WHERE nav_date >= '{nav_start_str}' and nav_date <= '{nav_end_str}' order by fund_name asc, nav_date asc, fund_property asc
            """
        else:
            sql = f"""
            SELECT id, fund_name, fund_status, fund_property, nav_date, unit_nav, accumulative_nav from vw_fund_netvalue AS t1 WHERE nav_date = (SELECT MAX(t2.nav_date) from vw_fund_netvalue as t2 WHERE nav_date >= '{nav_start_str}' and nav_date <= '{nav_end_str}' and  t1.fund_name = t2.fund_name and t1.product_property_id = t2.product_property_id) order by fund_name asc, nav_date asc, fund_property asc
            """
        # 将读出来的结果直接变为dataframe
        data_rows = pd.read_sql(sql=sql, con=conn, coerce_float=True)
        # 过滤investor_id和contract
        if not data_rows.empty:
            if instance_id is not None:
                data_rows = data_rows[data_rows['id'].isin(list_instance_id)]
            if fund_name is not None:
                data_rows = data_rows[data_rows['fund_name'].isin(list_fund_name)]
            if fund_property is not None:
                data_rows = data_rows[data_rows['fund_property'].isin(list_fund_property)]
        return data_rows
    except Exception as ex:
        print('读取官网净值数据失败:\r\n{}'.format(traceback.format_exc()))


def get_custody_netvalue_data(instance_id=None, fund_code=None, fund_name=None, nav_start=constant.DATETIME_MIN,
                              nav_end=constant.DATETIME_MAX, db_string=None, dataframe=True, is_latest=True):
    """
    获取托管净值同步数据
    :param instance_id:
    :param fund_code:
    :param fund_name:
    :param nav_start:
    :param nav_end:
    :param db_string:
    :param dataframe:
    :return:
    """
    data_df_dict = dict()
    instance_id = get_list_parameter(para=instance_id, para_type=(str, int))
    fund_name = get_list_parameter(para=fund_name, para_type=str)
    fund_code = get_list_parameter(para=fund_code, para_type=str)
    db_string = constant.DB_OPERATION_RD_ENGINE_STRING if db_string is None else db_string
    db_model = db_oper_models.t_vw_operation_custody_netvalue_sync
    try:
        session = get_db_session(db_string)
        if not is_latest:
            q = session.query(db_model)
        else:
            q = session.query(db_model.columns['product_code'], func.max(db_model.columns['nav_date']))
        conditions = None
        if instance_id is not None:
            conditions = db_oper_models.t_vw_operation_custody_netvalue_sync.columns['id'] == instance_id
        if fund_name is not None:
            if conditions is None:
                conditions = db_oper_models.t_vw_operation_custody_netvalue_sync.columns['product_name'].in_(fund_name)
            else:
                conditions = and_(conditions,
                                  db_oper_models.t_vw_operation_custody_netvalue_sync.columns['product_name'].in_(
                                      fund_name))
        if fund_code is not None:
            if conditions is None:
                conditions = db_oper_models.t_vw_operation_custody_netvalue_sync.columns['product_code'].in_(fund_code)
            else:
                conditions = and_(conditions,
                                  db_oper_models.t_vw_operation_custody_netvalue_sync.columns['product_code'].in_(
                                      fund_code))
        # 过滤净值日期
        if not is_latest:
            if conditions is None:
                conditions = and_(db_oper_models.t_vw_operation_custody_netvalue_sync.columns['nav_date'] >= nav_start,
                                  db_oper_models.t_vw_operation_custody_netvalue_sync.columns['nav_date'] <= nav_end)
            else:
                conditions = and_(conditions,
                                  db_oper_models.t_vw_operation_custody_netvalue_sync.columns['nav_date'] >= nav_start,
                                  db_oper_models.t_vw_operation_custody_netvalue_sync.columns['nav_date'] <= nav_end)
            q = q.order_by(desc(db_oper_models.t_vw_operation_custody_netvalue_sync.columns['nav_date']))
            data_list = q.all() if conditions is None else q.filter(conditions).all()
            data_list = [{key: getattr(item, key) for key in item._fields} for item in data_list]
            data_df = pd.DataFrame.from_dict(data_list)
        else:
            if conditions is not None:
                grouped_result = q.filter(conditions).group_by(db_model.columns['product_code'],
                                                               db_model.columns['product_name']).all()
            else:
                grouped_result = q.group_by(db_model.columns['product_code'], db_model.columns['product_name']).all()
            latest_nav_date_list = list()
            nav_data_list = list()
            # 统计当前所有基金的最新净值
            for each_pair in grouped_result:
                latest_nav_date_list.append(each_pair[1])
            # 如果不为空，则获取最小时间
            if len(latest_nav_date_list) > 0:
                min_nav_date = min(latest_nav_date_list)
                # 以最小时间单位查询数据
                if conditions is None:
                    q = session.query(db_model).filter(db_model.columns['nav_date'] >= min_nav_date)
                else:
                    q = session.query(db_model).filter(and_(conditions, db_model.columns['nav_date'] >= min_nav_date))
                nav_data_list = [{key: getattr(item, key) for key in item._fields} for item in q.all()]
            data_df = pd.DataFrame.from_dict(nav_data_list)
            if not data_df.empty:
                data_df = data_df.sort_values(by=['product_code', 'nav_date'], ascending=[True, False])
                latest_data_list = list()
                # 筛选出最新的数据
                for each_product_code, each_nav_df in data_df.groupby(by=['product_code', 'product_name']):
                    latest_data_list.append(
                        each_nav_df[each_nav_df['nav_date'] == each_nav_df['nav_date'].max()])
                data_df = pd.concat(latest_data_list, sort=True)
        if dataframe:
            return data_df
        else:
            if not data_df.empty:
                for product_code, each_nav_df in data_df.groupby(by=['stockfullcode']):
                    nav_df_copy = each_nav_df.copy()
                    nav_df_copy.reset_index(inplace=True, drop=True)
                    data_df_dict[product_code] = nav_df_copy
                return data_df_dict
    except Exception as ex:
        print('读取产品净值数据失败:\r\n{}'.format(traceback.format_exc()))


def get_fund_info_data(instance_id=None, fund_name=None, fund_code=None, db_string=None, dataframe=True):
    """
    获取产品的风控数据，需要从运营系统中获取
    :param instance_id:
    :param fund_name:
    :param fund_code:
    :param db_string:
    :param dataframe:
    :return:
    """
    instance_id = get_list_parameter(para=instance_id, para_type=(str, int))
    fund_name = get_list_parameter(para=fund_name, para_type=str)
    fund_code = get_list_parameter(para=fund_code, para_type=str)
    db_string = constant.DB_OPERATION_RD_ENGINE_STRING if db_string is None else db_string
    try:
        session = get_db_session(db_string)
        q = session.query(db_oper_models.t_vw_operation_product_info)
        conditions = None
        if instance_id is not None:
            conditions = db_oper_models.t_vw_operation_product_info.columns['id'] == instance_id
        if fund_name is not None:
            if conditions is None:
                conditions = db_oper_models.t_vw_operation_product_info.columns['name'].in_(fund_name)
            else:
                conditions = and_(conditions, db_oper_models.t_vw_operation_product_info.columns['name'].in_(fund_name))
        if fund_code is not None:
            if conditions is None:
                conditions = db_oper_models.t_vw_operation_product_info.columns['product_code'].in_(fund_code)
            else:
                conditions = and_(conditions,
                                  db_oper_models.t_vw_operation_product_info.columns['product_code'].in_(fund_code))
        q = q.order_by(asc(db_oper_models.t_vw_operation_product_info.columns['name']))
        data_list = q.all() if conditions is None else q.filter(conditions).all()
        data_list = [{key: getattr(item, key) for key in item._fields} for item in data_list]
        if dataframe:
            data_df = pd.DataFrame.from_dict(data_list)
            return data_df
        else:
            return data_list
    except Exception as ex:
        print('读取产品风控数据失败:\r\n{}'.format(traceback.format_exc()))


def get_risk_mgt_info_data(instance_id=None, account_category=None, account_name=None, account_category_code=None,
                           investor_id=None, group_name=None, group_code=None, db_string=None, dataframe=True):
    instance_id = get_list_parameter(para=instance_id, para_type=(str, int))
    investor_id = get_list_parameter(para=investor_id, para_type=str)
    account_name = get_list_parameter(para=account_name, para_type=str)
    account_category = get_list_parameter(para=account_category, para_type=str)
    account_category_code = get_list_parameter(para=account_category_code, para_type=str)
    group_name = get_list_parameter(para=group_name, para_type=str)
    group_code = get_list_parameter(para=group_code, para_type=str)
    db_string = constant.DB_OPERATION_RD_ENGINE_STRING if db_string is None else db_string
    try:
        session = get_db_session(db_string)
        q = session.query(db_oper_models.t_vw_operation_risk_management)
        conditions = None
        if instance_id is not None:
            conditions = db_oper_models.t_vw_operation_risk_management.columns['id'] == instance_id
        if investor_id is not None:
            if conditions is None:
                conditions = db_oper_models.t_vw_operation_risk_management.columns['investor_id'].in_(investor_id)
            else:
                conditions = and_(conditions,
                                  db_oper_models.t_vw_operation_risk_management.columns['investor_id'].in_(investor_id))
        if account_name is not None:
            if conditions is None:
                conditions = db_oper_models.t_vw_operation_risk_management.columns['account_name'].in_(account_name)
            else:
                conditions = and_(conditions,
                                  db_oper_models.t_vw_operation_risk_management.columns['account_name'].in_(
                                      account_name))
        if group_name is not None:
            if conditions is None:
                conditions = db_oper_models.t_vw_operation_risk_management.columns['group_name'].in_(group_name)
            else:
                conditions = and_(conditions,
                                  db_oper_models.t_vw_operation_risk_management.columns['group_name'].in_(group_name))
        if group_code is not None:
            if conditions is None:
                conditions = db_oper_models.t_vw_operation_risk_management.columns['group_code'].in_(group_code)
            else:
                conditions = and_(conditions,
                                  db_oper_models.t_vw_operation_risk_management.columns['group_code'].in_(group_code))
        if account_category is not None:
            if conditions is None:
                conditions = db_oper_models.t_vw_operation_risk_management.columns['account_category'].in_(
                    account_category)
            else:
                conditions = and_(conditions,
                                  db_oper_models.t_vw_operation_risk_management.columns['account_category'].in_(
                                      account_category))
        if account_category_code is not None:
            if conditions is None:
                conditions = db_oper_models.t_vw_operation_risk_management.columns['account_category_code'].in_(
                    account_category_code)
            else:
                conditions = and_(conditions,
                                  db_oper_models.t_vw_operation_risk_management.columns['account_category_code'].in_(
                                      account_category_code))
        q = q.order_by(asc(db_oper_models.t_vw_operation_risk_management.columns['investor_id']))
        data_list = q.all() if conditions is None else q.filter(conditions).all()
        data_list = [{key: getattr(item, key) for key in item._fields} for item in data_list]
        if dataframe:
            data_df = pd.DataFrame.from_dict(data_list)
            return data_df
        else:
            return data_list
    except Exception as ex:
        print('读取账号风控数据失败:\r\n{}'.format(traceback.format_exc()))


def calculate_investor_netvalue(data_df):
    # 获取data_df中账号的净值 nav = balance / initial_balance
    if data_df.empty:
        return data_df
    investor_id_list = data_df['investor_id'].unique().tolist()
    # 去掉组记录的风控记录
    investor_id_list = [item for item in investor_id_list if item is not None]

    pass


def get_individual_netvalue_data(investor_id=None, start_dt=None, end_dt=None, is_latest=True):
    """
    获取单账号的风控数据, 包括预警和净值
    :param investor_id:
    :param account_name:
    :param start_dt:
    :param end_dt:
    :param is_latest:
    :return:
    """
    result_df = pd.DataFrame()
    trading_account_data = get_trading_account_data(investor_id=investor_id, start_date=start_dt, end_date=end_dt,
                                                    is_latest=is_latest)
    if not trading_account_data.empty:
        # 重命名做merge
        rename_columns(data_df=trading_account_data, columns_map={'username': 'investor_id'})
        # drop
        drop_columns(data_df=trading_account_data, columns=['account_name'])
        # 排序
        trading_account_data.sort_values(by=['investor_id', 'date_trading'], inplace=True)

        # 如果不是获取最新的净值，由balance自己推算历史净值曲线
        data_list = list()
        # 按照账号分开计算
        for each_investor_id, each_investor_nv_df in trading_account_data.groupby(by=['investor_id']):
            each_nv_df_copy = each_investor_nv_df.copy()
            each_nv_df_copy.reset_index(inplace=True)
            # 求净值增长率
            # 真实的balance = 当前balance + 结算单出入金(带符号)
            each_nv_df_copy['real_balance'] = each_nv_df_copy['balance'] + each_nv_df_copy['settlement_dw']
            # 对real_balance做0值处理，避免出现后续求profit_pct的时候分母为0的情况
            each_nv_df_copy['real_balance'] = np.where(each_nv_df_copy['real_balance'] == 0, 1,
                                                       each_nv_df_copy['real_balance'])
            each_nv_df_copy['last_balance'] = each_nv_df_copy['balance'].shift(1)
            each_nv_df_copy.fillna({'last_balance': each_nv_df_copy['balance']}, inplace=True)
            each_nv_df_copy['last_balance'] = np.where(each_nv_df_copy['last_balance'].isnull(),
                                                       each_nv_df_copy['real_balance'],
                                                       each_nv_df_copy['last_balance'])
            # 对last_balance做0值处理，避免出现后续求profit_pct的时候分母为0的情况
            each_nv_df_copy['last_balance'] = np.where(each_nv_df_copy['last_balance'] == 0, 1,
                                                       each_nv_df_copy['last_balance'])
            each_nv_df_copy['profit_pct'] = (each_nv_df_copy['profit'] / each_nv_df_copy['last_balance']).astype(
                np.float)
            each_nv_df_copy.at[0, 'profit_pct'] = 0
            each_nv_df_copy['profit_pct_accum'] = each_nv_df_copy['profit_pct'].cumsum()
            each_nv_df_copy['profit_pct'] = each_nv_df_copy['profit_pct'] + 1
            each_nv_df_copy['profit_pct_accum'] = each_nv_df_copy['profit_pct_accum'] + 1
            data_list.append(each_nv_df_copy)
        result_df = pd.concat(data_list, sort=True)
        # 做NA填充
        result_df.fillna({'balance': -1, 'date_trading': constant.DATETIME_MAX}, inplace=True)
        # 重命名
        rename_columns(data_df=result_df,
                       columns_map={'account_name': 'product_name', 'date_trading': 'tday', 'profit_pct': 'nav',
                                    'profit_pct_accum': 'nav_accum'})
    return result_df


def get_individual_risk_mgt_data(investor_id=None, account_name=None, start_dt=None, end_dt=None, is_latest=True):
    """
    获取单账号的风控数据, 包括预警和净值
    :param investor_id:
    :param account_name:
    :param start_dt:
    :param end_dt:
    :param is_latest:
    :return:
    """
    result_df = pd.DataFrame()
    # 读取账号风控配置记录
    risk_data_df = get_risk_mgt_info_data(investor_id=investor_id, account_name=account_name)
    # 读取账号的结算单数据
    if risk_data_df.empty:
        return risk_data_df
    # TODO 需要解析investor_id为None的实际账号
    risk_data_df.dropna(subset=['investor_id'], inplace=True)
    list_investor_id = sorted(risk_data_df['investor_id'].tolist())
    trading_account_data = get_trading_account_data(investor_id=list_investor_id, start_date=start_dt, end_date=end_dt,
                                                    is_latest=is_latest)
    if not trading_account_data.empty:
        # 重命名做merge
        rename_columns(data_df=trading_account_data, columns_map={'username': 'investor_id'})
        # drop
        drop_columns(data_df=trading_account_data, columns=['account_name'])
        # 排序
        trading_account_data.sort_values(by=['investor_id', 'date_trading'], inplace=True)

        # 如果不是获取最新的净值，由balance自己推算历史净值曲线
        if not is_latest:
            data_list = list()
            # 按照账号分开计算
            for each_investor_id, each_investor_nv_df in trading_account_data.groupby(by=['investor_id']):
                each_nv_df_copy = each_investor_nv_df.copy()
                each_nv_df_copy.reset_index(inplace=True)
                # 求净值增长率
                # 真实的balance = 当前balance + 结算单出入金(带符号)
                each_nv_df_copy['real_balance'] = each_nv_df_copy['balance'] + each_nv_df_copy['settlement_dw']
                # 对real_balance做0值处理，避免出现后续求profit_pct的时候分母为0的情况
                each_nv_df_copy['real_balance'] = np.where(each_nv_df_copy['real_balance'] == 0, 1,
                                                           each_nv_df_copy['real_balance'])
                each_nv_df_copy['last_balance'] = each_nv_df_copy['balance'].shift(1)
                each_nv_df_copy.fillna({'last_balance': each_nv_df_copy['balance']}, inplace=True)
                each_nv_df_copy['last_balance'] = np.where(each_nv_df_copy['last_balance'].isnull(),
                                                           each_nv_df_copy['real_balance'],
                                                           each_nv_df_copy['last_balance'])
                # 对last_balance做0值处理，避免出现后续求profit_pct的时候分母为0的情况
                each_nv_df_copy['last_balance'] = np.where(each_nv_df_copy['last_balance'] == 0, 1,
                                                           each_nv_df_copy['last_balance'])
                each_nv_df_copy['profit_pct'] = (each_nv_df_copy['profit'] / each_nv_df_copy['last_balance']).astype(
                    np.float)
                each_nv_df_copy.at[0, 'profit_pct'] = 0
                each_nv_df_copy['profit_pct_accum'] = each_nv_df_copy['profit_pct'].cumsum()
                each_nv_df_copy['profit_pct'] = each_nv_df_copy['profit_pct'] + 1
                each_nv_df_copy['profit_pct_accum'] = each_nv_df_copy['profit_pct_accum'] + 1
                data_list.append(each_nv_df_copy)
            trading_account_data = pd.concat(data_list, sort=True)

        # 做风控数据和结算单数据的merge
        result_df = pd.merge(left=risk_data_df, right=trading_account_data, left_on='investor_id',
                             right_on='investor_id', how='left')
        # 筛选列
        filter_columns = ['investor_id', 'account_name', 'date_trading', 'balance', 'initial_balance', 'precaution']
        if not is_latest:
            filter_columns.extend(['profit_pct', 'profit_pct_accum'])
        result_df = result_df[filter_columns]
        # 做NA填充
        result_df.fillna({'balance': -1, 'date_trading': constant.DATETIME_MAX}, inplace=True)
        # 做initial_balance填充
        result_df['initial_balance'] = np.where(result_df['initial_balance'] <= 0, result_df['balance'],
                                                result_df['initial_balance'])
        # 结算净值
        if is_latest:
            result_df['nav'] = (result_df['balance'] / result_df['initial_balance'].astype(np.float)).astype(
                np.float).round(4)
            result_df['nav_accum'] = result_df['nav']
        # 构造product_code,为了和产品风控数据做merge
        result_df['product_code'] = result_df['account_name']
        # 重命名
        rename_columns(data_df=result_df,
                       columns_map={'account_name': 'product_name', 'date_trading': 'tday', 'profit_pct': 'nav',
                                    'profit_pct_accum': 'nav_accum'})
        # 去掉columns
        drop_columns(data_df=result_df, columns=['initial_balance'])
    return result_df


def get_fund_investor_risk_mgt_data(fund_name=None, fund_code=None, start_dt=None, end_dt=None, is_latest=True,
                                    account_type=0):
    """
    获取产品账户的风控数据，包括预警和净值
    :param fund_name:
    :param fund_code:
    :param start_dt:
    :param end_dt:
    :param is_latest:
    :param account_type:
    :return:
    """
    # 先获取基金和账号的关联
    fund_investor_data = get_fund_investor_data(fund_name=fund_name, fund_code=fund_code,
                                                security_type_num=account_type)
    fund_investor_data = fund_investor_data[
        ['investor_id', 'product_name', 'product_code', 'security_type', 'security_company_name']].copy()
    # 再获取基金的基本信息，主要是获取产品的风控数据
    fund_info_data = get_fund_info_data(fund_name=fund_name, fund_code=fund_code)
    rename_columns(data_df=fund_info_data, columns_map={'name': 'product_name'})
    # 获取最新托管数据的产品净值
    fund_custody_netvalue_data = get_custody_netvalue_data(fund_code=fund_code, fund_name=fund_name, nav_start=start_dt,
                                                           nav_end=end_dt, is_latest=is_latest)
    drop_columns(data_df=fund_custody_netvalue_data,
                 columns=['id', 'update_date', 'custody_code', 'custody_name', 'create_date', 'fund_type',
                          'id_custody'])
    # 做净值数据和产品风控数据的merge
    fund_info_summary_data = pd.DataFrame()
    if not fund_info_data.empty and not fund_custody_netvalue_data.empty:
        fund_info_summary_data = pd.merge(left=fund_info_data, right=fund_custody_netvalue_data,
                                          left_on=['product_code', 'product_name'],
                                          right_on=['product_code', 'product_name'],
                                          how='left')
        # fund_info_summary_data.dropna(subset=['nav', 'nav_date'], inplace=True)
        # 没有打通托管净值的数据用-1代替
        fund_info_summary_data.fillna({'nav': -1, 'nav_date': constant.DATETIME_MAX}, inplace=True)
    if not fund_investor_data.empty and not fund_info_summary_data.empty:
        fund_info_summary_data = pd.merge(left=fund_investor_data, right=fund_info_summary_data,
                                          left_on=['product_code', 'product_name'],
                                          right_on=['product_code', 'product_name'], how='left')
        fund_info_summary_data.drop_duplicates(subset=['investor_id', 'product_code', 'product_name', 'nav_date'],
                                               inplace=True)
    if not fund_info_summary_data.empty:
        fund_info_summary_data = fund_info_summary_data[
            ['investor_id', 'product_code', 'product_name', 'precaution', 'nav', 'accumulative_nav', 'nav_date',
             'asset_nav']]
        # 重命名
        rename_columns(data_df=fund_info_summary_data,
                       columns_map={'asset_nav': 'balance', 'nav_date': 'tday', 'accumulative_nav': 'nav_accum'})
    return fund_info_summary_data


def get_adjust_summarized_risk_mgt_data(start_dt=None, end_dt=None, is_latest=True):
    # 获取已打通托管的风控数据
    org_data = get_summarized_risk_mgt_data(start_dt=start_dt, end_dt=end_dt, is_latest=is_latest)
    org_data['tday'] = pd.to_datetime(org_data['tday'])
    return_data = org_data.copy()
    # 过滤出需要单独计算的产品及关联账户，目前是飒露紫和启明一号
    lack_of_ind_risk_data_df = org_data[(org_data['tday'] == constant.DATETIME_MAX) & (org_data['nav'] == -1)]
    if not lack_of_ind_risk_data_df.empty:
        # 提取产品对应的预警线数据
        query_investor_id_list = lack_of_ind_risk_data_df['investor_id'].unique().tolist()
        # 单独计算飒露紫和启明一号所对应的账号的净值数据
        extra_product_nav_df = get_individual_netvalue_data(investor_id=query_investor_id_list, start_dt=start_dt,
                                                            end_dt=end_dt, is_latest=is_latest)
        if not extra_product_nav_df.empty:
            # 将预警线数据做合并
            extra_product_nav_df = pd.merge(left=extra_product_nav_df, right=lack_of_ind_risk_data_df[
                ['investor_id', 'product_code', 'product_name', 'precaution']], left_on=['investor_id'],
                                            right_on=['investor_id'], how='left')
            extra_product_nav_df = extra_product_nav_df[
                ['balance', 'investor_id', 'nav', 'nav_accum', 'precaution', 'product_code', 'product_name', 'tday']]
        # 删除lack_of_ind_risk_data_df
        if not extra_product_nav_df.empty:
            org_data = org_data[~((org_data['tday'] == constant.DATETIME_MAX) & (org_data['nav'] == -1))].copy()
            # 拼接extra_product_nav_df
            return_data = pd.concat([org_data, extra_product_nav_df], sort=True)
            return_data.reset_index(drop=True, inplace=True)
    return return_data


def get_summarized_risk_mgt_data(instance_name=None, instance_code=None, start_dt=None, end_dt=None, is_latest=True,
                                 account_type=0):
    # 默认获取3个月
    default_start_tday = pd.to_datetime(dt.datetime.now() - relativedelta(months=3)).strftime('%Y-%m-%d')
    default_end_dt = pd.to_datetime(dt.datetime.now()).strftime('%Y-%m-%d')
    start_dt = default_start_tday if start_dt is None and not is_latest else start_dt
    end_dt = default_end_dt if end_dt is None and not is_latest else end_dt
    result_df = pd.DataFrame()
    concat_list = list()
    # 产品账户的风控数据
    fund_investor_risk_mgt_data = get_fund_investor_risk_mgt_data(fund_name=instance_name, fund_code=instance_code,
                                                                  start_dt=start_dt, end_dt=end_dt, is_latest=is_latest,
                                                                  account_type=account_type)
    # 获取单账户的风控数据
    individual_risk_mgt_data = get_individual_risk_mgt_data(investor_id=instance_code, account_name=instance_name,
                                                            start_dt=start_dt, end_dt=end_dt, is_latest=is_latest)
    if not fund_investor_risk_mgt_data.empty:
        concat_list.append(fund_investor_risk_mgt_data)
    if not individual_risk_mgt_data.empty:
        concat_list.append(individual_risk_mgt_data)

    # 做分控数据merge
    if len(concat_list) > 0:
        result_df = pd.concat(concat_list, sort=True)
    return result_df


def rename_columns(data_df, columns_map):
    """
    重命名列
    :param data_df:
    :param columns_map:
    :return:
    """
    data_df.rename(columns=columns_map, inplace=True)


def drop_columns(data_df, columns=[]):
    """
    去掉无用的列
    :param data_df:
    :param columns:
    :return:
    """
    drop_columns_list = list()
    for item in columns:
        if item in data_df.columns:
            drop_columns_list.append(item)
    data_df.drop(drop_columns_list, inplace=True, axis=1)


class OperationFeedTest(unittest.TestCase):
    def setUp(self) -> None:
        pass

    def tearDown(self) -> None:
        pass

    def test_get_fund_account_data(self):
        fund_code = 'SQG349'
        data = get_fund_investor_data(fund_code=fund_code)
        pass

    def test_build_product_netvalue_data(self):
        investor_id_list = ['8001002518', '10008113', '88180049', '109090028', '28116633', '20001590', '1756009',
                            '86006122', '80380368']
        netvalue = 1.0568
        precaution = 0.88
        balance = 20710279.44
        tday = '2021-11-08'
        risk_data = pd.DataFrame(data=investor_id_list, columns=['investor_id'])
        risk_data['product_name'] = '宣夜投资大有一号私募证券投资基金'
        risk_data['tday'] = tday
        risk_data['netvalue'] = netvalue
        risk_data['precaution'] = precaution
        risk_data['balance'] = balance

    def test_get_fund_info_data(self):
        fund_info_df = get_fund_info_data()
        pass

    def test_get_custody_netvalue_data(self):
        data = get_custody_netvalue_data(is_latest=True)
        pass

    def test_get_custody_netvalue_data(self):
        data = get_custody_netvalue_data(nav_start='2021-01-01', nav_end='2021-12-31', is_latest=False)
        pass

    def test_get_fund_investor_info_data(self):
        # fund_name = '宣夜投资大有一号私募证券投资基金'
        start_dt = '2021-08-19'
        end_dt = '2021-11-19'
        fund_name = None
        # start_dt = None
        # end_dt = None
        is_latest = False
        fund_investor_info_data = get_summarized_risk_mgt_data(instance_name=fund_name, start_dt=start_dt,
                                                               end_dt=end_dt,
                                                               is_latest=is_latest)
        pass

    def test_get_individual_risk_mgt_data(self):
        data_df = get_individual_risk_mgt_data(start_dt='2021-05-01', end_dt='2021-12-31', is_latest=False)
        pass

    def test_get_individual_netvalue_data(self):
        data_df = get_individual_netvalue_data()
        pass

    def test_get_adjust_summarized_risk_mgt_data(self):
        today = dt.datetime.now().date()
        test_risk_data_all100 = get_adjust_summarized_risk_mgt_data(is_latest=True)
        test_risk_data_all100 = get_adjust_summarized_risk_mgt_data(
            start_dt=get_previous_tday(pd.to_datetime(today), 101).date(),
            end_dt=pd.to_datetime(today).date(), is_latest=False)
        pass

    def test_get_web_netvalue_data(self):
        range_web_netvalue_data = get_web_netvalue_data(is_latest=False)
        latest_web_netvalue_data = get_web_netvalue_data(is_latest=True)
