import re
import json
import redis
import pyodbc
import pandas as pd
import datetime as dt
import dateutil.parser as parser

DEFAULT_REDIS_STRING = 'redis://:redisxy123456R88@r-uf6fb5d61707be84pd.redis.rds.aliyuncs.com:6401'
BACKUP_REDIS_URL = 'redis://:redisxy123456R88@backupstrategynode.redis.rds.aliyuncs.com:6401'
DB_HIS_STRING = ';'.join([
    'DRIVER={MySQL ODBC 5.3 ANSI Driver}',
    'SERVER=rm-uf605nn2go0mj7q49o.mysql.rds.aliyuncs.com',
    'PORT=3306',
    'DATABASE=db_his',
    'UID=sqlro',
    'PWD=XYsqlrd654321a88'
])
MAIN_CONTRACT_CACHE = {}
SYMBOL_MARKET_CACHE = {}
SYMBOL_MULTIPLE_CACHE = {}
LIST_DATE_CACHE = {}
LEGAL_TRADE_TIME_CACHE = None
TDAY_CACHE = None


def _cache_main_contract(tday=None):
    global MAIN_CONTRACT_CACHE
    if tday is not None:
        today_str = tday.strftime('%Y-%m-%d')
    else:
        today_str = 'today'
    conn = pyodbc.connect(DB_HIS_STRING)
    cursor = conn.cursor()
    where_clause = "stockfullcode LIKE '%9001'"
    if today_str != 'today':
        where_clause += f"AND date_ed < '{today_str}'"
    sql = f"""
    SELECT * FROM 
        (SELECT stockfullcode, stockfullcode_main 
        FROM vw_edinfo 
        WHERE {where_clause}
        ORDER BY date_ed DESC) t 
    GROUP BY t.stockfullcode
    """
    MAIN_CONTRACT_CACHE[today_str] = {k: v for (k, v) in cursor.execute(sql)}


def get_future_main_contract(stockfullcode_list, tday=None):
    """
    查询9001合约对应的实际合约代码
    :param stockfullcode_list: 要查询的合约代码列表
    :param tday: 要查询交易日,不指定就查最新的
    :return: dict: key:9001合约代码,value:实际合约代码
    """
    global MAIN_CONTRACT_CACHE
    if isinstance(stockfullcode_list, str):
        stockfullcode_list = (stockfullcode_list,)
    if tday is None:
        tday_str = 'today'
        contract_dict = MAIN_CONTRACT_CACHE.get(tday_str, None)
    else:
        tday = parser.parse(tday)
        tday_str = tday.strftime('%Y-%m-%d')
        contract_dict = MAIN_CONTRACT_CACHE.get(tday_str, None)
    if contract_dict is None:
        _cache_main_contract(tday)
    contract_dict = MAIN_CONTRACT_CACHE[tday_str]
    return {k: v for (k, v) in contract_dict.items() if k in stockfullcode_list}


def _cache_future_info(symbol_list=None):
    global MAIN_CONTRACT_CACHE, SYMBOL_MULTIPLE_CACHE, LIST_DATE_CACHE, SYMBOL_MARKET_CACHE
    if len(MAIN_CONTRACT_CACHE) <= 0:
        _cache_main_contract()
    main_contract_dict = MAIN_CONTRACT_CACHE['today']
    quote = "'"
    if symbol_list is None:
        where_clause = f"stockfullcode IN ({','.join(quote + code + quote for code in main_contract_dict.values())})"
    else:
        if len(SYMBOL_MARKET_CACHE) <= 0:
            _cache_future_info()
        code_list = []
        for sym in symbol_list:
            if len(sym) <= 2:
                code_list.append(f'{SYMBOL_MARKET_CACHE[sym]}{sym}9001')
            else:
                code_list.append(sym)
        where_clause = f"stockfullcode IN ({','.join(quote + code + quote for code in code_list)})"
    conn = pyodbc.connect(DB_HIS_STRING)
    cursor = conn.cursor()
    sql = f"""
    SELECT stockfullcode, symbol, open_date, expire_date, volume_multiple 
    FROM vw_lookup_future_info
    WHERE {where_clause}
    """
    data_rows = cursor.execute(sql)
    sm_re = re.compile(r'^(\D{2})(\D+)')
    multiple_dict = dict()
    list_date_dict = dict()
    symbol_market_dict = dict()
    for row in data_rows:
        m = sm_re.match(row.stockfullcode)
        multiple_dict[m.group(2)] = float(row.volume_multiple)
        list_date_dict[row.stockfullcode] = (row.open_date, row.expire_date)
        symbol_market_dict[m.group(2)] = m.group(1)
    SYMBOL_MULTIPLE_CACHE.update(multiple_dict)
    LIST_DATE_CACHE.update(list_date_dict)
    SYMBOL_MARKET_CACHE.update(symbol_market_dict)


def get_symbol_multiple(symbol_list):
    """
    查询品种合约乘数
    :param symbol_list: str or iterable, 品种代码或9001代码均可,如rb, SQrb9001
    :return: dict
    """
    global SYMBOL_MULTIPLE_CACHE, SYMBOL_MARKET_CACHE
    if isinstance(symbol_list, str):
        symbol_list = (symbol_list,)
    sym_dict = {}
    sym_re = re.compile(r'(?:^\D{2})(\D+)')
    for symbol in symbol_list:
        if symbol.endswith('9001'):
            m = sym_re.match(symbol)
            sym_dict[m.group(1)] = symbol
        else:
            sym_dict[symbol] = symbol
    missing_symbol = set()
    for sym in sym_dict.keys():
        multi = SYMBOL_MULTIPLE_CACHE.get(sym, None)
        if not multi:
            missing_symbol.add(sym)
    if len(missing_symbol) > 0:
        _cache_future_info(missing_symbol)
    return {sym_dict[symbol]: multiple for symbol, multiple in SYMBOL_MULTIPLE_CACHE.items() if
            symbol in sym_dict.keys()}


def get_contract_list_date(code_list):
    """
    查询合约挂牌交易开始结束日期
    :param code_list: str or iterable, 合约代码列表, 如['SFIF2010', 'SQrb2101']
    :return: tuple(open_date, expire_date), 上市日, 最后交易日
    """
    global LIST_DATE_CACHE
    if isinstance(code_list, str):
        code_list = (code_list,)
    missing_code = set()
    for code in code_list:
        list_date = LIST_DATE_CACHE.get(code, None)
        if not list_date:
            missing_code.add(code)
    if len(missing_code) > 0:
        _cache_future_info(missing_code)
    return {code: (start, end) for code, (start, end) in LIST_DATE_CACHE.items() if code in code_list}


def _cache_legal_trade_time(cache_path='redis'):
    global LEGAL_TRADE_TIME_CACHE
    if not LEGAL_TRADE_TIME_CACHE:
        LEGAL_TRADE_TIME_CACHE = dict()
    if cache_path == 'redis':
        redis_url = DEFAULT_REDIS_STRING
    elif cache_path == 'redis_backup':
        redis_url = BACKUP_REDIS_URL
    else:
        return None
    client = redis.from_url(redis_url, decode_responses=True)
    json_str = client.get('SYMBOLTRADINGPERIOD')
    orig_dict = json.loads(json_str)
    orig_dict.pop('WXni')
    for sym, v in orig_dict.items():
        sym = sym[2:]
        LEGAL_TRADE_TIME_CACHE.setdefault(sym, [])
        for legal_time in v:
            date_start = parser.parse(legal_time['date_start'])
            date_end = parser.parse(legal_time['date_end'])
            time_start = parser.parse(legal_time['time_start']).time()
            time_end = parser.parse(legal_time['time_end']).time()
            LEGAL_TRADE_TIME_CACHE[sym].append({
                'date_start': date_start,
                'date_end': date_end,
                'time_start': time_start,
                'time_end': time_end})


def get_legal_trade_time(symbol_list, tday=None, cache_path='redis'):
    """
    查询品种交易时段
    :param symbol_list: str or iterable, 品种代码, 如['rb', 'i', 'IF']
    :param cache_path: str, 'redis' or 'redis_backup'
    :return: 交易时段开始结束时间
    """
    global LEGAL_TRADE_TIME_CACHE
    if not LEGAL_TRADE_TIME_CACHE or len(LEGAL_TRADE_TIME_CACHE) <= 0:
        _cache_legal_trade_time(cache_path)
    if not tday:
        tday = dt.datetime.today()
    else:
        if isinstance(tday, str):
            tday = parser.parse(tday)
    res_dict = {}
    for sym in symbol_list:
        res_dict.setdefault(sym, [])
        time_list = LEGAL_TRADE_TIME_CACHE.get(sym, None)
        if not time_list:
            continue
        for legal_time in time_list:
            if legal_time['date_start'] <= tday < legal_time['date_end']:
                res_dict[sym].append((legal_time['time_start'], legal_time['time_end']))
    return res_dict


def _cache_tday_list():
    global TDAY_CACHE
    if not TDAY_CACHE:
        TDAY_CACHE = list()
    conn = pyodbc.connect(DB_HIS_STRING)
    cursor = conn.cursor()
    sql = f"""
    SELECT tday FROM lookup_tday ORDER BY tday ASC
    """
    TDAY_CACHE = [pd.to_datetime(item[0]).date() for item in cursor.execute(sql)]


def get_tday_df(start_tday=None, end_tday=None):
    global TDAY_CACHE
    if not TDAY_CACHE:
        _cache_tday_list()
    start_tday = dt.datetime(1900, 1, 1).date() if start_tday is None else pd.to_datetime(start_tday).date()
    end_tday = dt.datetime(2079, 6, 6).date() if end_tday is None else pd.to_datetime(end_tday).date()
    tday_list = [tday for tday in TDAY_CACHE if start_tday <= tday <= end_tday]
    tday_df = pd.DataFrame(data=tday_list, columns=['tday'])
    return tday_df


if __name__ == '__main__':
    # 主力实际合约代码
    # print(get_future_main_contract('SQrb9001'))
    # print(get_future_main_contract(['SQrb9001', 'ZZAP9001']))
    # print(get_future_main_contract(['SQrb9001', 'ZZAP9001'], tday='2020-01-01'))
    # _cache_tday_list()
    get_tday_df(start_tday='2011-01-01', end_tday='2013-01-01')
    get_tday_df(start_tday='2015-01-01', end_tday='2019-01-01')

    # 合约乘数
    print(get_symbol_multiple('fu'))
    print(get_symbol_multiple(['rb', 'IF', 'SFIF9001']))
    print(get_symbol_multiple(['AP', 'au', 'hc', 'sc']))
    # 合约挂牌交易开始结束日期
    print(get_contract_list_date('DLi2101'))
    print(get_contract_list_date(['SQrb2101', 'ZZTA101']))
    # 品种交易时段
    print(get_legal_trade_time(['IC', 'IF', 'rb']))
    print(get_legal_trade_time(['IC', 'IF', 'rb'], tday='2015-01-01'))
    pass
