# -*- coding: utf-8 -*-
"""转化JQData Panel数据格式成为平板格式"""
import os
import numpy as np
import pandas as pd
import datetime as dt
import tushare as ts
import unittest
import logging
import h5py as h5
import jqdatasdk as jqsdk
from rio3.base import constant
from rio3.base.constant import ZQ_KLINE_ORG_COLUMNS_DTYPES_DICT, ZQ_KLINE_FORMAT_COLUMNS, H5_JQ_COMMON_DIR
from rio3.log.setup import setup_screen_logger, setup_file_logger
from jqdatasdk import *
from rio3.base.constant import H5_JQ_KLINE_DIR, H5_JQ_DAILY_DIR, H5_JQ_TKLINE_DIR, ZQ_KLINE_ORG_COLUMNS, DATETIME_MIN, \
    DATETIME_MAX
from rio3.datafeed.research.tushareIO import _get_trading_days, _get_tday_list, _get_previous_tday, _get_next_tday
from rio3.datafeed.research.jqdata_sdkIO import normalize_code_to_local, normalize_code_to_jq, DEFAULT_ACCOUNT_NAME, \
    DEFAULT_PASSWORD, get_security_info
from rio3.datafeed.common import _get_cache_files_dir, convert_freq2num, _get_jq_future_symbol, \
    _get_security_type_by_market, _get_market, _get_future_symbol
from rio3.datafeed.feed import _get_history_data_org, convert_httpjq2zq_format, _get_trading_datetime_range, \
    get_history_data, _get_options_data_org
from rio3.datafeed.dbIO import get_stock_id
from rio3.utils.filesys import list_files

setup_file_logger()
logger = logging.getLogger('root')
TDAY_DIVIDE_UNIT = 100


def _get_tdays_by_interval(trading_days_list):
    """
    将Tday按照TDAY_DIVIDE_UNIT切分
    :param trading_days_list:
    :return:
    """
    interval_tday_list = list()
    start_index = 0
    tdays_interval_length, remainder = divmod(len(trading_days_list), TDAY_DIVIDE_UNIT)
    # 循环归纳出TDAY_DIVIDE_UNIT单位的tday数据
    for i in range(tdays_interval_length):
        end_index = start_index + TDAY_DIVIDE_UNIT
        each_interval_list = trading_days_list[start_index: end_index]
        start_index = end_index
        interval_tday_list.append(each_interval_list)
    # 处理剩下的天数
    rest_tdays_list = trading_days_list[-remainder:]
    interval_tday_list.append(rest_tdays_list)
    return interval_tday_list


def get_jq_api(username=None, password=None):
    """
    获取默认jqsdk
    :return: 返回注册过的jqsdk
    """
    username = username if username is not None else DEFAULT_ACCOUNT_NAME
    password = password if password is not None else DEFAULT_PASSWORD
    if not jqsdk.is_auth():
        jqsdk.auth(username, password)
        logger.info(f"get_jq_api接口使用账号:'{username}'")
    return jqsdk


def convert_stockcode_jq2zq_data(start_date, end_date, stockfullcode, freqs=None, fields=None, export_dir=None,
                                 append=True, replace=False, skip_check=False):
    list_stockfullcode = [stockfullcode, ] if isinstance(stockfullcode, str) else stockfullcode
    list_fields = [fields, ] if isinstance(fields, str) else fields
    list_freqs = [freqs, ] if isinstance(freqs, str) else freqs
    jq_fields = ['open', 'close', 'high', 'low', 'volume', 'money', 'factor', 'high_limit', 'low_limit', 'avg',
                 'pre_close', 'open_interest']
    # 转stockcode格式
    list_stockfullcode = normalize_code_to_jq(list_stockfullcode)
    start_date_str = dt.datetime.strftime(pd.to_datetime(start_date), '%Y-%m-%d %H:%M:%S')
    end_date_str = dt.datetime.strftime(pd.to_datetime(end_date), '%Y-%m-%d %H:%M:%S')
    total_number = 0
    for each_stockcode in list_stockfullcode:
        # 先获取交易日daily的数据，主要是为了获取后复权下的factor
        daily_df = _get_history_data_org(stockfullcode=each_stockcode, start_date=start_date_str,
                                         end_date=end_date_str, fields=jq_fields, freq='1d', ed_type='post',
                                         cache_path='http')
        local_stockcode = normalize_code_to_local(each_stockcode)
        security_type = _get_security_type_by_market(_get_market(local_stockcode))
        for freq in list_freqs:
            # 从JQData获取缓存数据
            freq_stockcode_df = _get_history_data_org(stockfullcode=each_stockcode, start_date=start_date_str,
                                                      end_date=end_date_str, fields=jq_fields, freq=freq, ed_type=None,
                                                      cache_path='http')
            # 如果行情源数据和factor Daily的数据都不为空
            if not daily_df.empty and not freq_stockcode_df.empty:
                # 将Daily的Factor数据Merge到行情源数据
                daily_factor_df = daily_df[['today', 'factor']]
                freq_stockcode_df = pd.merge(left=freq_stockcode_df, right=daily_factor_df, left_on='today',
                                             right_on='today')
                # 重新整理columns, 重命名和去除不需要的列
                freq_stockcode_df.rename(columns={'factor_y': 'factor'}, inplace=True)
                freq_stockcode_df.drop(['factor_x'], inplace=True, axis=1)
                zq_kline_df = _get_contract(kline_df=freq_stockcode_df, jq_stockcode=each_stockcode,
                                            security_type=security_type)
                # 排序和过滤掉不需要缓存的列
                zq_kline_df = zq_kline_df[ZQ_KLINE_ORG_COLUMNS]
                # 定义格式
                zq_kline_df = zq_kline_df.astype(ZQ_KLINE_ORG_COLUMNS_DTYPES_DICT, copy=True)
                logger.info(
                    u"共获取交易日:'{}-{}', 品种: '{}', 级别:'{}', 共'{}'条数据".format(start_date_str, end_date_str, local_stockcode,
                                                                          freq, len(zq_kline_df)))

                total_number += zq_df_to_h5(zq_kline_df=zq_kline_df, stockfullcode=local_stockcode,
                                            freq=freq,
                                            export_file_path=export_dir, append=append, replace=replace,
                                            skip_check=skip_check)
    return total_number


def convert_jq2zq_daily_data(security_type, start_date=None, end_date=None, freqs=None, fields=None, source_dir=None,
                             export_dir=None, append=True, replace=False, skip_check=False, underlying_name=None):
    """
    将日更数据拼接为研究平板数据
    :param security_type: 下载数据类型 stock/index/futures/sw/conbond
    :param start_date: 开始时间
    :param end_date: 结束时间
    :param freqs: 需要生成的数据频率
    :param fields: 需要拼接的field
    :param source_dir: 日更文件目录
    :param export_dir: 导出根目录
    :param append: 是否持续写入
    :param replace: 是否用现有的数据替换已经存在的数据，主要用于修复
    :param skip_check: 是否跳过检查，是否进行断点续传
    :param underlying_name: 期权的标的名称
    :return:
    """
    list_fields = [fields, ] if isinstance(fields, str) else fields
    list_freqs = [freqs, ] if isinstance(freqs, str) else freqs
    # 目前只支持按照security_type转化
    # 验证security_type
    if security_type not in ['conbond', 'futures', 'index', 'stock', 'sw', 'options']:
        raise TypeError(f'不支持的转化类型: {security_type}')
    # 日更数据的根目录
    source_dir = source_dir if source_dir is not None else H5_JQ_DAILY_DIR
    type_source_dir = os.path.join(source_dir, security_type)
    # 根据给定时间区间获取交易日
    trading_days = _get_trading_days(start_date=start_date, end_date=end_date)
    trading_days_range_list = _get_tdays_by_interval(trading_days_list=trading_days)
    for tday_range in trading_days_range_list:
        if security_type in ['index', 'stock']:
            convert_stock_data(security_type=security_type, list_tday=tday_range, list_freqs=list_freqs,
                               list_fields=list_fields, security_type_root_dir=type_source_dir, export_dir=export_dir,
                               append=append, replace=replace, skip_check=skip_check)
        elif security_type == 'futures':
            convert_futures_data(security_type=security_type, list_tday=tday_range, list_freqs=list_freqs,
                                 list_fields=list_fields, security_type_root_dir=type_source_dir, export_dir=export_dir,
                                 append=append, replace=replace, skip_check=skip_check)
        elif security_type in ['sw', 'conbond']:
            convert_plane_data(security_type=security_type, tday=tday_range, freqs=freqs,
                               security_type_root_dir=type_source_dir, export_dir=export_dir, append=append,
                               replace=replace)
        elif security_type == 'options':
            convert_options_data(security_type=security_type, list_tday=tday_range, list_freqs=list_freqs,
                                 list_fields=list_fields, underlying_name=underlying_name,
                                 security_type_root_dir=type_source_dir, export_dir=export_dir, append=append,
                                 replace=replace, skip_check=skip_check)


def convert_to_zq_h5(jq_kline_df, jq_stock, freq, tday, security_type, export_dir=None, append=True, replace=False):
    """
    转化成ZQ格式数据和本地标准代码名称，并最后存储成H5
    :param jq_kline_df:
    :param jq_stock:
    :param freq:
    :param tday:
    :param security_type:
    :param export_dir:
    :param append:
    :param replace:
    :return:
    """
    converted_num = 0
    # 过滤N/A数据
    jq_kline_df = jq_kline_df.dropna(axis=0, how='any')
    if not jq_kline_df.empty:
        # 转化jq格式数据为zq格式
        zq_kline_df = convert_cachedjq2zq_format(jq_kline_df, freq_str=freq, tday=tday,
                                                 security_type=security_type)
        # 转化JQ格式的代码为本地格式
        local_stock_code = normalize_code_to_local(jq_stock)
        # 按照代码名称存储H5文件，返回转化条目数
        converted_num = zq_df_to_h5(zq_kline_df=zq_kline_df, stockfullcode=local_stock_code, freq=freq,
                                    export_file_path=export_dir, append=append, replace=replace)
    return converted_num


def _get_replaced_merge_data(replace_data, stockfullcode, freq):
    merged_data = pd.DataFrame()
    current_data = _get_history_data_org(stockfullcode=stockfullcode, start_date=DATETIME_MIN, end_date=DATETIME_MAX,
                                         freq=freq)
    # 如果replace_date不是空，则进行替换操作
    if not current_data.empty and not replace_data.empty:
        # 构造DateTrade数据
        current_data['DateTrade'] = pd.to_datetime(current_data['date'] * 1000000 + current_data['ets'],
                                                   format='%Y%m%d%H%M%S')
        replace_data['DateTrade'] = pd.to_datetime(replace_data['date'] * 1000000 + replace_data['ets'],
                                                   format='%Y%m%d%H%M%S')
        # 先删掉原始数据
        current_data = current_data[~current_data.today.isin(replace_data.today)]
        # 拼接新数据
        merged_data = pd.concat([current_data, replace_data])
        # 按照DateTrade排序
        merged_data = merged_data.sort_values(by=['DateTrade'])
        merged_data.drop(['DateTrade'], inplace=True, axis=1)
        merged_data['symbol'] = np.where(merged_data['symbol'].isnull(), stockfullcode, merged_data['symbol'])
    return merged_data


def _whether_update_data(latest_zq_kline_df, stockfullcode, freq, security_type=None):
    """
    根据传入的新数据时间戳，判断是否需要更新现有数据
    :param latest_zq_kline_df:
    :param stockfullcode:
    :param freq:
    :param security_type:
    :return:
    """
    whether_update = False
    latest_first_tday_num = int(latest_zq_kline_df.iloc[0].today)
    if security_type == 'options':
        current_data = _get_options_data_org(stockfullcode=stockfullcode, start_date=DATETIME_MIN,
                                             end_date=DATETIME_MAX, freq=freq)
    else:
        current_data = _get_history_data_org(stockfullcode=stockfullcode, start_date=DATETIME_MIN,
                                             end_date=DATETIME_MAX, freq=freq)
    # 如果当前数据不为空，则判断更新时间戳
    if not current_data.empty:
        # 判断current_da的最后一条记录的tday的记录是不是大于等于给定的end，如果是，则无需更新
        last_bar_tday_num = int(current_data.iloc[-1].today)
        # 如果不是，则重新制定获取更新数据的开始和结束时间
        # TODO 日更新的判断有问题，这里如果latest_zq_kline_df的起始区间小于现有缓存数据，但是结束时间大于现有缓存数据，是需要有一个合并过程
        if last_bar_tday_num < latest_first_tday_num:
            whether_update = True
    # 如果没有数据，则需要进行数据生成
    else:
        whether_update = True
    return whether_update


def zq_df_to_h5(zq_kline_df, stockfullcode, freq, export_file_path=None, data_columns=['date', 'today', 'sts', 'ets'],
                append=True, replace=False, skip_check=False, security_type=None):
    """
    缓存成ZQ格式
    :param zq_kline_df:
    :param stockfullcode:
    :param freq:
    :param export_file_path:
    :param data_columns:
    :param append: 持续添加
    :param replace:
    :param skip_check:
    :param security_type:
    :return: -1代表转存失败，正数代表转化成功
    """
    assert isinstance(zq_kline_df, pd.DataFrame)
    insert_update_num = 0
    # 如果没有给定的export_file_path或者给定的是文件夹，则按照指定路径格式生成到NAS上
    # 根据period构造文件后缀名
    export_file_component = r'{}\{}_{}.h5'
    file_suffix = 'tkline' if convert_freq2num(freq_str=freq) < 60 else 'kline'
    if export_file_path is None:
        # 如果秒数小于60，需要修改默认导出路径
        export_dir = H5_JQ_TKLINE_DIR if convert_freq2num(freq_str=freq) < 60 else H5_JQ_KLINE_DIR
        # 处理期权数据的时候,需要指定security_type,因为合约与股票/期货的市场有重名,所以必须区分
        if security_type == 'options':
            cache_file_dir = _get_cache_files_dir(cache_root_path=export_dir, stockfullcode=stockfullcode,
                                                  is_local=False, pass_security_type=security_type)
        # 处理非期权数据,直接按照stockfullcode的市场名做判断
        else:
            cache_file_dir = _get_cache_files_dir(cache_root_path=export_dir, stockfullcode=stockfullcode,
                                                  is_local=False)
        export_file_path = export_file_component.format(cache_file_dir, stockfullcode, file_suffix)
    else:
        if not os.path.exists(export_file_path):
            os.mkdir(export_file_path)
        if os.path.isdir(export_file_path):
            export_file_path = export_file_component.format(export_file_path, stockfullcode, file_suffix)
    # 检查文件目录是否已经被生成
    export_file_dir = os.path.dirname(export_file_path)
    if not os.path.exists(export_file_dir):
        os.makedirs(export_file_dir)
    # 根据period生成groupset名字
    node_name = freq.lower()
    # 如果不是replace数据或者skip_check是跳过检查，则正常检查最新的数据时间戳，判断是否需要更新和断点更新
    if not replace:
        whether_update = True if skip_check else _whether_update_data(latest_zq_kline_df=zq_kline_df,
                                                                      stockfullcode=stockfullcode,
                                                                      freq=freq, security_type=security_type)
        if whether_update:
            # 获取更新数据的today
            zq_kline_df_tday_start = int(zq_kline_df.iloc[0].today)
            zq_kline_df_tday_end = int(zq_kline_df.iloc[-1].today)
            try:
                zq_kline_df.to_hdf(export_file_path, node_name, complevel=9, complib='blosc', format='table',
                                   data_columns=data_columns, append=append)
                insert_update_num = len(zq_kline_df)
                logger.info(
                    u"[PASS]: 交易日 '{}-{}', 品种'{}', 级别'{}', 共导入'{}'条，存储路径: {} ".format(zq_kline_df_tday_start,
                                                                                      zq_kline_df_tday_end,
                                                                                      stockfullcode,
                                                                                      freq, insert_update_num,
                                                                                      export_file_path))
            except Exception as ex:
                insert_update_num = -1
                logger.error(
                    u"[ERROR]: 交易日 '{}-{}', 品种：'{}', 级别：'{}', 存储路径: {}, 数据生成失败 {}，".format(zq_kline_df_tday_start,
                                                                                           zq_kline_df_tday_end,
                                                                                           stockfullcode, freq,
                                                                                           export_file_path,
                                                                                           ex))
        else:
            logger.info(
                u"[NO ACTION]: 品种'{}', 级别'{}', 当前已经是最新记录, 无需更新，存储路径: {} ".format(stockfullcode, freq,
                                                                                 export_file_path))
    # 如果是replace，则用最新的数据替换掉原始数据，然后和现有的数据进行merge,然后重新刷入文件
    else:
        merge_data = _get_replaced_merge_data(replace_data=zq_kline_df, stockfullcode=stockfullcode, freq=freq)
        if not merge_data.empty:
            # 排序
            merge_data = merge_data[ZQ_KLINE_ORG_COLUMNS]
            # 定义格式
            merge_data = merge_data.astype(ZQ_KLINE_ORG_COLUMNS_DTYPES_DICT, copy=True)
            if not merge_data.empty:
                # 获取更新数据的today
                zq_kline_df_tday_start = int(zq_kline_df.iloc[0].today)
                zq_kline_df_tday_end = int(zq_kline_df.iloc[-1].today)
                try:
                    merge_data.to_hdf(export_file_path, node_name, complevel=9, complib='blosc', format='table',
                                      data_columns=data_columns)
                    insert_update_num = len(zq_kline_df)
                    logger.info(
                        u"[PASS]: 交易日 '{}-{}', 品种'{}', 级别'{}', 更新'{}'条，存储路径: {} ".format(zq_kline_df_tday_start,
                                                                                         zq_kline_df_tday_end,
                                                                                         stockfullcode, freq,
                                                                                         insert_update_num,
                                                                                         export_file_path))
                except Exception as ex:
                    insert_update_num = -1
                    logger.error(
                        u"[ERROR]: 交易日 '{}-{}', 品种：'{}', 级别：'{}', 存储路径: {}, 数据更新失败 {}，".format(zq_kline_df_tday_start,
                                                                                               zq_kline_df_tday_end,
                                                                                               stockfullcode, freq,
                                                                                               export_file_path,
                                                                                               ex))
        else:
            logger.info(
                u"[WARNING]: 品种'{}', 级别'{}', 没有数据需要更新，存储路径: {} ".format(stockfullcode, freq,
                                                                        insert_update_num,
                                                                        export_file_path))
    return insert_update_num


def convert_options_data(security_type, underlying_name, list_tday, list_fields, list_freqs, security_type_root_dir,
                         export_dir=None, append=True, replace=False, skip_check=False):
    """
    生成期权合约缓存
    :param security_type:
    :param underlying_name:
    :param list_tday:
    :param list_fields:
    :param list_freqs:
    :param security_type_root_dir: 日更新缓存数据的文件目录 stock/future/index/options
    :param export_dir:
    :param append:
    :param replace:
    :param skip_check:
    :return:
    """
    total_number = 0
    tday_freq_df_list = list()
    logger.info(
        u"开始获取交易日:'{}-{}', 类型:'{}', 级别:'{}'的field数据".format(list_tday[0], list_tday[-1], security_type,
                                                            ','.join(list_freqs)))
    for tday in list_tday:
        # 期权的日线存储目录格式为： H5_JQ_DAILY_DIR + underlying_name + tday(%Y%m%d)
        type_tday_root_dir = os.path.join(security_type_root_dir, underlying_name, tday)
        for freq in list_freqs:
            field_column_df = pd.DataFrame()
            for field in list_fields:
                single_field_path = os.path.join(type_tday_root_dir, freq, '.'.join([field, 'h5']))
                # 初始化stockcode字典，key为品种，value为fields字段的展开数据
                if h5.is_hdf5(single_field_path):
                    field_df = pd.read_hdf(path_or_buf=single_field_path)
                    # stack会默认去掉NA数据，为了保证数据长度一致性，设置dropna=False
                    field_df_stack = field_df.stack(dropna=False).reset_index(name=field)
                    field_df_stack = field_df_stack[field_df_stack['level_1'] != 'datetrade']
                    if field == list_fields[0]:
                        field_column_df = field_df_stack.copy()
                    else:
                        field_column_df[field] = field_df_stack[field]
            if not field_column_df.empty:
                field_column_df['freq'] = freq
                field_column_df['tday'] = int(tday)
                tday_freq_df_list.append(field_column_df)
    total_kline_concat_df = pd.concat(tday_freq_df_list) if len(tday_freq_df_list) > 0 else pd.DataFrame()
    # 统一缓存这一个tday时间线中所有品种的数据
    if not total_kline_concat_df.empty:
        if 'level_1' in total_kline_concat_df.columns or 'level_0' in total_kline_concat_df.columns:
            # 新的商品期权用的是get_bars获取行情的方式，level_0字段变为date，# 如果存在level_0，则变为date_trade, 如果存在date，也改为date_trade
            total_kline_concat_df.rename(columns={'level_1': 'contract', 'level_0': 'date_trade', 'date': 'date_trade'},
                                         inplace=True)
            logger.info(
                u"共获取交易日:'{}-{}', 级别:'{}', 共'{}'条数据".format(list_tday[0], list_tday[-1], ','.join(list_freqs),
                                                            len(total_kline_concat_df)))
            total_number = bulk_convert_to_zq_h5(contract_kline_df=total_kline_concat_df, security_type=security_type,
                                                 export_dir=export_dir, append=append, replace=replace,
                                                 skip_check=skip_check)
        else:
            logger.warning(u"交易日'{}'，类型:{}, 缺少拼接关键字段:'level_0'和'level_1'".format(','.join(list_tday), security_type))
    else:
        logger.info(
            u"[NO DATA]: 交易日:'{}-{}', 类型:'{}', 级别:'{}' 没有field数据".format(list_tday[0], list_tday[-1], security_type,
                                                                         ','.join(list_freqs)))
    return total_number


def convert_stock_data(security_type, list_tday, list_fields, list_freqs, security_type_root_dir, export_dir=None,
                       append=True, replace=False, skip_check=False):
    """
    根据时间切片范围，获取fields数据，用于拼接成面板数据
    :param security_type:
    :param list_tday:
    :param list_fields:
    :param list_freqs:
    :param security_type_root_dir:
    :param export_dir:
    :param append:
    :param replace:
    :param skip_check:
    :return:
    """
    total_number = 0
    tday_freq_df_list = list()
    logger.info(
        u"开始获取交易日:'{}-{}', 类型:'{}', 级别:'{}'的field数据".format(list_tday[0], list_tday[-1], security_type,
                                                            ','.join(list_freqs)))
    for tday in list_tday:
        type_tday_root_dir = os.path.join(security_type_root_dir, tday)
        # index/stock的日更存储格式相同 security_type/yyyymmdd/period/field.h5
        for freq in list_freqs:
            field_column_df = pd.DataFrame()
            for field in list_fields:
                single_field_path = os.path.join(type_tday_root_dir, freq, '.'.join([field, 'h5']))
                # 初始化stockcode字典，key为品种，value为fields字段的展开数据
                if h5.is_hdf5(single_field_path):
                    field_df = pd.read_hdf(path_or_buf=single_field_path, key='data')
                    # stack会默认去掉NA数据，为了保证数据长度一致性，设置dropna=False
                    field_df_stack = field_df.stack(dropna=False).reset_index(name=field)
                    field_df_stack = field_df_stack[field_df_stack['level_1'] != 'datetrade']
                    if field == list_fields[0]:
                        field_column_df = field_df_stack.copy()
                    else:
                        field_column_df[field] = field_df_stack[field]
            if not field_column_df.empty:
                field_column_df['freq'] = freq
                field_column_df['tday'] = int(tday)
                tday_freq_df_list.append(field_column_df)
    total_kline_concat_df = pd.concat(tday_freq_df_list) if len(tday_freq_df_list) > 0 else pd.DataFrame()
    # 统一缓存这一个tday时间线中所有品种的数据
    if not total_kline_concat_df.empty:
        if 'level_1' in total_kline_concat_df.columns and 'level_0' in total_kline_concat_df.columns:
            total_kline_concat_df.rename(columns={'level_1': 'contract', 'level_0': 'date_trade'}, inplace=True)
            logger.info(
                u"共获取交易日:'{}-{}', 级别:'{}', 共'{}'条数据".format(list_tday[0], list_tday[-1], ','.join(list_freqs),
                                                            len(total_kline_concat_df)))
            total_number = bulk_convert_to_zq_h5(contract_kline_df=total_kline_concat_df, security_type=security_type,
                                                 export_dir=export_dir, append=append, replace=replace,
                                                 skip_check=skip_check)
        else:
            logger.warning(u"交易日'{}'，类型:{}, 缺少拼接关键字段:'level_0'和'level_1'".format(','.join(list_tday), security_type))
    else:
        logger.info(
            u"[NO DATA]: 交易日:'{}-{}', 类型:'{}', 级别:'{}' 没有field数据".format(list_tday[0], list_tday[-1], security_type,
                                                                         ','.join(list_freqs)))
    return total_number


def _get_contract(kline_df, jq_stockcode, security_type):
    """
    # 如果是9999的主力合约，则需要获取当前品种在该交易日的标的合约，不然就等于自己
    :param kline_df:
    :param jq_stockcode:
    :param security_type:
    :return:
    """
    if security_type == 'futures' and '9999' in jq_stockcode:
        if not kline_df.empty:
            tday_list = kline_df.today.unique().tolist()
            tday_contract_list = list()
            # 获取当前kline交易日所对应的标的合约列表
            for tday in tday_list:
                jq_symbol = _get_jq_future_symbol(jq_stockfullcode=jq_stockcode)
                # dominant_future_code = _http_get_dominant_future(code=jq_symbol,
                #                                                  date=pd.to_datetime(tday, format='%Y%m%d').strftime(
                #                                                      '%Y-%m-%d'))
                dominant_future_code = get_jq_api().get_dominant_future(underlying_symbol=jq_symbol,
                                                                        date=pd.to_datetime(tday,
                                                                                            format='%Y%m%d').strftime(
                                                                            '%Y-%m-%d'))
                tday_contract_list.append([tday, normalize_code_to_local(dominant_future_code)])
            tday_contract_df = pd.DataFrame(data=tday_contract_list, columns=['today', 'dominant_contract'])
            # # 对tday_contract_df进行dominant_contract groupby, 处理换约的边界记录，在旧主力合约的第一天添加当个交易日本合约和新主力合约的收盘价
            # tday_contract_df['next_dominant_cotract'] = tday_contract_df['dominant_contract'].shift(-1)
            # tday_ed_contract_df = tday_contract_df[fe
            #     tday_contract_df['dominant_contract'] != tday_contract_df['next_dominant_cotract']].copy()
            # # 循环获取换约点，获取price
            # for row in tday_ed_contract_df.itertuples():
            #     tday_str = pd.to_datetime(row.today, format='%Y%m%d').strftime('%Y-%m-%d')
            #     current_dominant_contract_kine_df = get_jq_api().get_price(
            #         security=normalize_code_to_jq(row.dominant_contract),
            #         start_date=tday_str, end_date=tday_str,
            #         frequency='1d')
            #     next_dominant_cotract_kine_df = get_jq_api().get_price(
            #         security=normalize_code_to_jq(row.next_dominant_cotract),
            #         start_date=tday_str, end_date=tday_str,
            #         frequency='1d')
            #     current_domiant_close_price = current_dominant_contract_kine_df.close.iloc[0] if not current_dominant_contract_kine_df.empty else None
            #     next_domiant_close_price = next_dominant_cotract_kine_df.close.iloc[
            #         0] if not next_dominant_cotract_kine_df.empty else None
            # 对K线数据进行merge
            if not tday_contract_df.empty:
                kline_df = pd.merge(left=kline_df, right=tday_contract_df, how='left', left_on='today',
                                    right_on='today')
                kline_df['dominant_contract'] = np.where(kline_df['dominant_contract'] != '',
                                                         kline_df['dominant_contract'],
                                                         kline_df['contract'])
                kline_df.drop(['contract'], inplace=True, axis=1)
                kline_df.rename(columns={'dominant_contract': 'contract'}, inplace=True)
    return kline_df


def bulk_convert_to_zq_h5(contract_kline_df, security_type, export_dir=None, append=True, replace=False,
                          skip_check=False):
    """
    批量更新多个品种的Kline缓存文件
    :param contract_kline_df:
    :param security_type:
    :param export_dir:
    :param append:
    :param replace:
    :param skip_check:
    :return:
    """
    total_number = 0
    for stockfullcode, org_stock_kline_df in contract_kline_df.groupby('contract'):
        stock_kline_df = org_stock_kline_df.copy()
        # 拿SFIF9001做实验
        # if stockfullcode != '510050.XSHG':
        #     continue
        # 过滤数据
        _fill_na(stock_kline_df)
        for freq, freq_kline_df in stock_kline_df.groupby('freq'):
            if not freq_kline_df.empty:
                local_stockcode = normalize_code_to_local(stockfullcode)
                zq_kline_df = convert_cachedjq2zq_format(jq_kline_df=freq_kline_df, freq_str=freq,
                                                         security_type=security_type)
                zq_kline_df = _get_contract(kline_df=zq_kline_df, jq_stockcode=stockfullcode,
                                            security_type=security_type)
                kline_date_start = zq_kline_df.iloc[0].today
                kline_date_end = zq_kline_df.iloc[-1].today
                logger.info(
                    u"共拼接交易日:'{}-{}', 品种: '{}', 级别:'{}', 共'{}'条数据".format(kline_date_start, kline_date_end,
                                                                          local_stockcode, freq,
                                                                          len(zq_kline_df)))
                total_number += zq_df_to_h5(zq_kline_df=zq_kline_df, stockfullcode=local_stockcode,
                                            freq=freq, export_file_path=export_dir, append=append, replace=replace,
                                            skip_check=skip_check, security_type=security_type)
        # 每次更新数据都同时更新一遍ed_factor的因子
        if security_type == 'futures' and '9999' in stockfullcode:
            cache_future_main_ed_info_data(stockfullcode=normalize_code_to_local(stockfullcode), export_dir=export_dir)
    return total_number


def _fill_na(quote_df):
    # 对于期权标的数据，open_interest为NA，需要赋值为-1
    if 'open_interest' in quote_df.columns:
        quote_df.fillna({'open_interest': -1}, inplace=True)
    quote_df.dropna(axis=0, how='any', inplace=True)


def cache_future_main_ed_info_data(stockfullcode, export_dir=None, cache_path='nas'):
    full_tday_df = _get_tday_list()
    full_tday_df.rename(columns={'date': 'next_tday'}, inplace=True)
    # 根据最新的1d的kline，来做ed_info列表的更新
    main_contract_local_df = get_history_data(stockfullcode=stockfullcode, start_date=DATETIME_MIN,
                                              end_date=DATETIME_MAX, freq='1d', cache_path=cache_path)
    if cache_path == 'http':
        main_contract_local_df = _get_contract(kline_df=main_contract_local_df,
                                               jq_stockcode=normalize_code_to_jq(stockfullcode),
                                               security_type='futures')
    # 构造更新合约的组合
    main_contract_local_df['next_main_contract'] = main_contract_local_df['contract'].shift(-1)
    # 筛选出交替的合约
    deliver_contract_df = main_contract_local_df[
        main_contract_local_df['contract'] != main_contract_local_df['next_main_contract']].copy()
    # 获取当前kline交易日所对应的标的合约列表
    # 对tday_contract_df进行dominant_contract groupby, 处理换约的边界记录，在旧主力合约的最后一天添加当个交易日本合约和新主力合约的收盘价
    # 循环获取换约点，获取price
    current_contract_close_price_list = list()
    next_contract_close_price_list = list()
    deliver_contract_df.dropna(how='any', inplace=True)
    for row in deliver_contract_df.itertuples():
        tday_str = pd.to_datetime(row.today, format='%Y%m%d').strftime('%Y-%m-%d')
        current_dominant_contract_kine_df = get_jq_api().get_price(
            security=normalize_code_to_jq(row.contract),
            start_date=tday_str, end_date=tday_str,
            frequency='1d', fq=None)
        next_dominant_cotract_kine_df = get_jq_api().get_price(
            security=normalize_code_to_jq(row.next_main_contract),
            start_date=tday_str, end_date=tday_str,
            frequency='1d', fq=None)
        current_domiant_close_price = current_dominant_contract_kine_df.close.iloc[
            0] if not current_dominant_contract_kine_df.empty else None
        next_domiant_close_price = next_dominant_cotract_kine_df.close.iloc[
            0] if not next_dominant_cotract_kine_df.empty else None
        current_contract_close_price_list.append(current_domiant_close_price)
        next_contract_close_price_list.append(next_domiant_close_price)
    deliver_contract_df['current_contract_close_price'] = current_contract_close_price_list
    deliver_contract_df['next_contract_close_price'] = next_contract_close_price_list
    # 拼接Next Tday
    total_contract_df = pd.merge(left=deliver_contract_df, right=full_tday_df, how='left', left_on='today',
                                 right_on='pretrade_date')
    # 筛选edinfo信息
    total_contract_df = total_contract_df[
        ['today', 'next_tday', 'open', 'close', 'high', 'low', 'pre_close', 'symbol', 'contract',
         'next_main_contract', 'current_contract_close_price', 'next_contract_close_price']]
    # 存入ed_info
    export_file_component = r'{}\{}_{}.h5'
    if export_dir is None:
        # 如果秒数小于60，需要修改默认导出路径
        cache_file_dir = _get_cache_files_dir(cache_root_path=H5_JQ_KLINE_DIR, stockfullcode=stockfullcode,
                                              is_local=False)
        export_file_path = export_file_component.format(cache_file_dir, stockfullcode, 'kline')
    else:
        if os.path.isdir(export_dir):
            export_file_path = export_file_component.format(export_dir, stockfullcode, 'kline')
    # 检查文件目录是否已经被生成
    export_file_dir = os.path.dirname(export_file_path)
    if not os.path.exists(export_file_dir):
        os.makedirs(export_file_dir)
    # 保存成h5
    try:
        total_contract_df.to_hdf(path_or_buf=export_file_path, key='edinfo', format='table', complevel=9,
                                 complib='blosc')
        logger.info(
            u"[缓存EDINFO成功]: 品种'{}' 共{}条 ".format(stockfullcode, len(total_contract_df)))
    except Exception as ex:
        logger.info(
            u"[缓存EDINFO出错]: 品种:'{}' 错误: {} ".format(stockfullcode, ex))


def convert_futures_data(security_type, list_tday, list_fields, list_freqs, security_type_root_dir, export_dir=None,
                         append=True, replace=False, skip_check=False):
    """
    转化期货数据
    :param security_type:
    :param list_tday:
    :param list_fields:
    :param list_freqs:
    :param security_type_root_dir:
    :param export_dir:
    :param append:
    :param replace:
    :param skip_check:
    :return:
    """
    total_number = 0
    tday_freq_df_list = list()
    logger.info(
        u"开始获取交易日:'{}-{}', 类型:'{}', 级别:'{}'的field数据".format(list_tday[0], list_tday[-1], security_type,
                                                            ','.join(list_freqs)))
    # 期货的日更数据按照tday+market+symbol+freq+fields存储
    for tday in list_tday:
        type_tday_root_dir = os.path.join(security_type_root_dir, tday)
        if not os.path.exists(type_tday_root_dir):
            continue
        market_dir_list = os.listdir(type_tday_root_dir)
        for market_dir in market_dir_list:
            # 拿中金所做实验先
            # if market_dir != 'CCFX':
            #     continue
            market_dir_path = os.path.join(type_tday_root_dir, market_dir)
            if not os.path.exists(market_dir_path):
                continue
            symbol_dir_list = os.listdir(market_dir_path)
            for symbol in symbol_dir_list:
                # #拿IF做实验
                # if symbol != 'IF':
                #     continue
                field_column_df = pd.DataFrame()
                for freq in list_freqs:
                    tday_market_symbol_freq_dir = os.path.join(market_dir_path, symbol, freq)
                    for field in list_fields:
                        single_field_path = os.path.join(tday_market_symbol_freq_dir, '.'.join([field, 'h5']))
                        if h5.is_hdf5(single_field_path):
                            field_df = pd.read_hdf(path_or_buf=single_field_path, key='data')
                            field_df_stack = field_df.stack(dropna=False).reset_index(name=field)
                            field_df_stack = field_df_stack[field_df_stack['level_1'] != 'datetrade']
                            if field == list_fields[0]:
                                field_column_df = field_df_stack.copy()
                            else:
                                field_column_df[field] = field_df_stack[field]
                    if not field_column_df.empty:
                        field_column_df['freq'] = freq
                        field_column_df['tday'] = int(tday)
                        tday_freq_df_list.append(field_column_df)
                    logger.info(
                        u"共获取交易日:'{}', 类型: '{}', 级别:'{}', 品种: '{}', 市场: '{}', 共'{}'条数据".format(tday, security_type,
                                                                                               freq, symbol, market_dir,
                                                                                               len(field_column_df)))
    total_kline_concat_df = pd.concat(tday_freq_df_list) if len(tday_freq_df_list) > 0 else pd.DataFrame()
    # 统一缓存这一个tday时间线中所有品种的数据
    if not total_kline_concat_df.empty:
        if 'level_1' in total_kline_concat_df.columns and 'level_0' in total_kline_concat_df.columns:
            total_kline_concat_df.rename(columns={'level_1': 'contract', 'level_0': 'date_trade'}, inplace=True)
            logger.info(
                u"共获取交易日:'{}-{}', 级别:'{}', 共'{}'条数据".format(list_tday[0], list_tday[-1], ','.join(list_freqs),
                                                            len(total_kline_concat_df)))
            total_number = bulk_convert_to_zq_h5(contract_kline_df=total_kline_concat_df, security_type=security_type,
                                                 export_dir=export_dir, append=append, replace=replace,
                                                 skip_check=skip_check)
        else:
            logger.warning(u"交易日'{}'，类型:{}, 缺少拼接关键字段:'level_0'和'level_1'".format(','.join(list_tday), security_type))
    else:
        logger.info(
            u"[NO DATA]: 交易日:'{}-{}', 类型:'{}', 级别:'{}' 没有field数据".format(list_tday[0], list_tday[-1], security_type,
                                                                         ','.join(list_freqs)))
    return total_number


def convert_plane_data(security_type, tday, freq, security_type_root_dir, export_dir=None, append=True, replace=False):
    total_number = 0
    fields_concat_list = list()
    type_tday_root_dir = os.path.join(security_type_root_dir, tday)
    # sw/的日更存储格式相同 security_type/yyyymmdd/period/field.h5
    daily_cache_file_format = '{}_{}.h5'
    if security_type == 'sw':
        daily_cache_file_name = daily_cache_file_format.format('sw_l1', tday)
        market_prefix = 'SW'
    elif security_type == 'conbond':
        daily_cache_file_name = daily_cache_file_format.format('conbond', tday)
        market_prefix = 'CB'
    daily_h5_file_path = os.path.join(type_tday_root_dir, freq, daily_cache_file_name)
    # 初始化stockcode字典，key为品种，value为fields字段的展开数据
    if h5.is_hdf5(daily_h5_file_path):
        daily_df = pd.read_hdf(path_or_buf=daily_h5_file_path, key='data')
        if not daily_df.empty:
            daily_df['code'] = (market_prefix + daily_df['code']).astype(np.str)
            for each_stock, stock_df in daily_df.groupby(daily_df.code):
                total_number += convert_to_zq_h5(jq_kline_df=stock_df, jq_stock=each_stock, freq=freq,
                                                 tday=tday, security_type=security_type, export_dir=export_dir,
                                                 append=append, replace=replace)
        logger.info(
            f"交易日:'{tday}', 类型: '{security_type}', 级别:'{freq}', 品种数量:'{len(daily_df.code.unique())}', "
            f"共导入'{total_number}'条")
    else:
        logger.info(
            f"交易日:'{tday}', 类型: '{security_type}', 级别:'{freq}', 没有field数据或者field不全，共导入'{total_number}'条")
    return total_number


def convert_cachedjq2zq_format(jq_kline_df, freq_str, security_type, kline_columns=ZQ_KLINE_ORG_COLUMNS):
    """
    转化单品种的JQKline数据为ZQ格式数据
    :param jq_kline_df:
    :param freq_str:
    :param tday:
    :param security_type:
    :return:
    """
    zq_kline_df = jq_kline_df.copy()
    # 申万1级和可转债行情目前的日线和future/stock不同，需要提前构造index
    if security_type not in ['futures', 'stock', 'index', 'options']:
        zq_kline_df.index = pd.to_datetime(zq_kline_df.date)
    span_value = convert_freq2num(freq_str=freq_str)
    zq_kline_df['span'] = int(span_value)
    dt_year = zq_kline_df.date_trade.dt.year
    dt_month = zq_kline_df.date_trade.dt.month
    dt_day = zq_kline_df.date_trade.dt.day
    dt_hour = zq_kline_df.date_trade.dt.hour
    dt_minute = zq_kline_df.date_trade.dt.minute
    if security_type not in ['futures', 'options']:
        zq_kline_df['date'] = dt_year * 10000 + dt_month * 100 + dt_day
        zq_kline_df['ets'] = dt_hour * 10000 + dt_minute * 100
        zq_kline_df['sts'] = np.where(zq_kline_df['freq'] == '1d', 0,
                                      np.where(zq_kline_df['ets'] % 10000 == 0, zq_kline_df['ets'] - 4100,
                                               zq_kline_df['ets'] - 100))
    else:
        zq_kline_df['date'] = zq_kline_df.date_trade.dt.strftime("%Y%m%d").astype(np.int64)
        zq_kline_df['ets'] = zq_kline_df.date_trade.dt.strftime('%H%M%S').astype(np.int64)
        zq_kline_df['sts'] = (zq_kline_df.date_trade - pd.Timedelta(seconds=int(span_value))).dt.strftime(
            '%H%M%S').astype(
            np.int64)
    # 除了期货以外，open_interest都设置为-1
    if security_type not in ['futures', 'options']:
        zq_kline_df['open_interest'] = -1
    if 'open_interest' not in zq_kline_df.columns:
        zq_kline_df['open_interest'] = -1
    # 对于open_interest是NA的数据，设置为-1
    zq_kline_df.fillna({'open_interest': -1}, inplace=True)
    # 除了期货和股票以外，factor都是1
    if security_type not in ['stock', 'futures']:
        zq_kline_df['factor'] = 1

    if security_type == 'options':
        zq_kline_df['high_limit'] = -1
        zq_kline_df['low_limit'] = -1
        zq_kline_df['pre_close'] = -1
        zq_kline_df['avg'] = -1

    # 将把header全部lowercase
    zq_kline_df.columns = zq_kline_df.columns.str.lower()

    # 修改column name
    zq_kline_df = zq_kline_df.rename(
        columns={'volume': 'vol', 'open_interest': 'oi', 'money': 'amount', 'tday': 'today'})

    # # 去除不需要的列
    # if 'avg' in zq_kline_df.columns:
    #     zq_kline_df.drop(['avg'], inplace=True, axis=1)

    # 将contract的jq格式转成标准模式
    if 'contract' in zq_kline_df.columns:
        zq_kline_df['contract'] = normalize_code_to_local(zq_kline_df['contract'].tolist())

    # 排序
    zq_kline_df = zq_kline_df[kline_columns]

    # 定义格式
    zq_kline_df = zq_kline_df.astype(ZQ_KLINE_ORG_COLUMNS_DTYPES_DICT, copy=True)

    return zq_kline_df


def concat_call_auction_data(jq_stockcode, jq_kline_df, start_date, end_date):
    """
    获取当前jq_kline_df的集合竞价数据，拼接上去
    :param jq_kline_df:
    :param start_date:
    :param end_date:
    :return:
    """
    if not jq_kline_df.empty:
        start_tday = jq_kline_df.iloc[0].today
        end_tday = jq_kline_df.iloc[-1].today
        call_auction_data = get_jq_api().get_call_auction(security=jq_stockcode, start_date=start_date,
                                                          end_date=end_date)


def extract_ed_info_data(stockfullcode, start, end, export_path=None):
    """
    获取ed_info_data
    :param stockfullcode:
    :param start:
    :param end:
    :param export_path:
    :return:
    """
    full_tday_df = _get_tday_list()
    full_tday_df.rename(columns={'date': 'next_tday'}, inplace=True)
    main_contract_local_df = get_history_data(stockfullcode=stockfullcode, start_date=start, end_date=end, freq='1d',
                                              cache_path=r'D:\TempCSV')
    main_contract_local_df['next_main_contract'] = main_contract_local_df['contract'].shift(-1)
    deliver_contract_df = main_contract_local_df[
        main_contract_local_df['contract'] != main_contract_local_df['next_main_contract']].copy()
    # 获取当前kline交易日所对应的标的合约列表
    # 对tday_contract_df进行dominant_contract groupby, 处理换约的边界记录，在旧主力合约的最后一天添加当个交易日本合约和新主力合约的收盘价
    # 循环获取换约点，获取price
    current_contract_close_price_list = list()
    next_contract_close_price_list = list()
    deliver_contract_df.dropna(how='any', inplace=True)
    for row in deliver_contract_df.itertuples():
        tday_str = pd.to_datetime(row.today, format='%Y%m%d').strftime('%Y-%m-%d')
        current_dominant_contract_kine_df = get_jq_api().get_price(
            security=normalize_code_to_jq(row.contract),
            start_date=tday_str, end_date=tday_str,
            frequency='1d')
        next_dominant_cotract_kine_df = get_jq_api().get_price(
            security=normalize_code_to_jq(row.next_main_contract),
            start_date=tday_str, end_date=tday_str,
            frequency='1d')
        current_domiant_close_price = current_dominant_contract_kine_df.close.iloc[
            0] if not current_dominant_contract_kine_df.empty else None
        next_domiant_close_price = next_dominant_cotract_kine_df.close.iloc[
            0] if not next_dominant_cotract_kine_df.empty else None
        current_contract_close_price_list.append(current_domiant_close_price)
        next_contract_close_price_list.append(next_domiant_close_price)
    deliver_contract_df['current_contract_close_price'] = current_contract_close_price_list
    deliver_contract_df['next_contract_close_price'] = next_contract_close_price_list
    # 拼接Next Tday
    total_contract_df = pd.merge(left=deliver_contract_df, right=full_tday_df, how='left', left_on='today',
                                 right_on='pretrade_date')
    total_contract_df = total_contract_df[
        ['today', 'next_tday', 'open', 'close', 'high', 'low', 'pre_close', 'symbol', 'contract', 'next_main_contract',
         'current_contract_close_price', 'next_contract_close_price']]
    total_contract_df.to_hdf(path_or_buf=export_path, key='edinfo', format='table', complevel=9, complib='blosc')
    return total_contract_df


def calculate_ed_info_by_oi(stockfullcode, export_path, start_date=DATETIME_MIN, end_date=DATETIME_MAX):
    """
    生成按照持仓量拼接的edinfo
    :param stockfullcode:
    :param export_path:
    :param start_date:
    :param end_date:
    :return:
    """
    tday_list = _get_trading_days(start_date=start_date, end_date=end_date)
    future_symbol = _get_future_symbol(stockfullcode=stockfullcode)
    # 9001代码的id_stock
    id_stock = get_stock_id(stockfullcode=stockfullcode)
    current_contract = ''
    ed_info_data_list = list()
    for tday in tday_list:
        tday_str = pd.to_datetime(tday).strftime('%Y-%m-%d')
        new_current_contract = get_jq_api().get_dominant_future(future_symbol.upper(), tday_str)
        if new_current_contract is None or new_current_contract == '':
            continue
        new_current_contract = normalize_code_to_local(new_current_contract)
        stockfullcode_main = normalize_code_to_local(new_current_contract)
        # 构建第一条ed_info记录
        if current_contract == '':
            id_main_stock = get_stock_id(stockfullcode=stockfullcode_main)
            date_ed = pd.to_datetime(_get_previous_tday(tday), format='%Y%m%d')
            date_execute = pd.to_datetime(tday)
            ed_info_data_list.append(
                [id_stock, stockfullcode, id_main_stock, stockfullcode_main, date_ed, date_execute, 0, 0, 0, 0, 0])
            logger.info(
                '初始化第一条edinfo: stockfullcode_main:{}, date_ed:{}, date_execute:{}, sgbl: {}'.format(stockfullcode_main,
                                                                                                    date_ed,
                                                                                                    date_execute, 0))
        # 从第二个交易日信息开始，查询合约是否一致，如果新合约不等于老合约，并且新合约大于老合约的合约数字，则进行更新
        else:
            if current_contract != '' and new_current_contract != current_contract:
                # 如果新合约数 > 老合约数
                if new_current_contract > current_contract:
                    # 此时需要获取上一个交易日的old和new的close_price
                    pre_tday = pd.to_datetime(_get_previous_tday(tday), format='%Y%m%d')
                    current_contract_kline_df = get_jq_api().get_price(
                        security=normalize_code_to_jq(current_contract),
                        start_date=pre_tday, end_date=pre_tday,
                        frequency='1d', fq=None)
                    new_contract_kline_df = get_jq_api().get_price(
                        security=normalize_code_to_jq(new_current_contract),
                        start_date=pre_tday, end_date=pre_tday,
                        frequency='1d', fq=None)
                    if not current_contract_kline_df.empty and not current_contract_kline_df.empty:
                        multiplier = (current_contract_kline_df.close.iloc[0] - new_contract_kline_df.close.iloc[
                            0]) / new_contract_kline_df.close.iloc[0]
                        multiplier = round(multiplier, 12)
                        date_execute = pd.to_datetime(tday, format='%Y%m%d')
                        date_ed = pd.to_datetime(pre_tday)
                        id_main_stock = get_stock_id(new_current_contract)
                        ed_info_data_list.append(
                            [id_stock, stockfullcode, id_main_stock, stockfullcode_main, date_ed, date_execute, 0, 0, 0,
                             multiplier,
                             0, current_contract_kline_df.close.iloc[0], new_contract_kline_df.close.iloc[0]])
                        logger.info(
                            'edinfog更新: stockfullcode_main:{}, date_ed:{}, date_execute:{}, sgbl: {}'.format(
                                new_current_contract,
                                date_ed,
                                date_execute, multiplier))
        current_contract = new_current_contract
    ed_info_df = pd.DataFrame(data=ed_info_data_list,
                              columns=['id_stock', 'stockfullcode', 'id_main_stock', 'stockfullcode_main',
                                       'date_ed', 'date_execute', 'fhbl', 'pgbl', 'pgjg', 'sgbl', 'preclose',
                                       'current_contract_close_price', 'new_contract_close_price'])
    return ed_info_df


def cache_security_info(types=[], cache_path=None):
    """
    缓存各类型的合约数据
    :param types:
    :param cache_path:
    :return:
    """
    # 获取security_df数据
    security_df = get_security_info(types=types, cache_path='http')
    # 按照type做分类缓存
    for each_type, type_security_df in security_df.groupby(security_df.type):
        cache_dir = constant.H5_JQ_COMMON_DIR if cache_path is None else cache_path
        export_dir = os.path.join(cache_dir, each_type)
        if not os.path.exists(export_dir):
            os.mkdir(export_dir)
        export_path = os.path.join(export_dir, 'contracts.h5')
        type_security_df.to_hdf(export_path, each_type, complevel=9, complib='blosc', format='table')


class TestJq2ZqIO(unittest.TestCase):
    def setUp(self):
        setup_screen_logger()
        setup_file_logger()
        self.logger = logging.getLogger('root')

    def tearDown(self):
        pass

    def test_cache_security_info(self):
        """
        缓存本地和JQ相匹配的合约信息缓存到H5
        :return:
        """
        type = ['futures', 'stock']
        cache_security_info(types=type)
        pass

    def test_convert_etf_options_zq_data(self):
        """
        Task:Option Step2: 更新JQData ETF期权合约的行情数据
        :return:
        """
        start = '2021-12-18'
        end = '2022-02-18'
        security_type = 'options'
        fields = ['open', 'close', 'high', 'low', 'volume', 'money', 'oi']
        list_freqs = ['1m', '5m', '15m', '1d']
        # list_freqs = ['1d']
        source_dir = r'\\nas\share\HDF5\JQData\Daily'
        export_dir = r'\\nas\share\HDF5\JQData\KlineH5\options\SH'
        # export_dir = r'D:\TempCSV\options_data\1d'
        list_underlying_name = ['50ETF']
        skip_check = True
        for underlying_name in list_underlying_name:
            for freqs in list_freqs:
                convert_jq2zq_daily_data(security_type=security_type, start_date=start, end_date=end, freqs=freqs,
                                         underlying_name=underlying_name, fields=fields, source_dir=source_dir,
                                         export_dir=export_dir, skip_check=skip_check)
        pass

    def test_convert_index_options_zq_data(self):
        start = '2021-11-19'
        end = '2021-12-18'
        security_type = 'options'
        fields = ['open', 'close', 'high', 'low', 'volume', 'money', 'open_interest']
        # source_dir = r'D:\TempCSV\HDF5\JQData\Daily'
        # export_dir = r'D:\TempCSV\HDF5\JQData\KlineH5\options\SF'
        source_dir = r'\\nas\share\HDF5\JQData\Daily'
        export_dir = r'\\nas\share\HDF5\JQData\KlineH5\options\SF'
        list_underlying_name = ['INDEX']
        # 先生成5m/15m，1分钟看需求
        list_freqs = ['5m', '15m', '1d']
        skip_check = True
        for underlying_name in list_underlying_name:
            for freqs in list_freqs:
                convert_jq2zq_daily_data(security_type=security_type, start_date=start, end_date=end, freqs=freqs,
                                         underlying_name=underlying_name, fields=fields, source_dir=source_dir,
                                         export_dir=export_dir, skip_check=skip_check)
        pass

    def test_convert_future_options_zq_data(self):
        start = '2019-11-01'
        end = '2022-12-31'
        security_type = 'options'
        fields = ['open', 'close', 'high', 'low', 'volume', 'money', 'open_interest']
        source_dir = r'\\nas\share\HDF5\JQData\Daily'
        # export_dir = r'D:\TempCSV\HDF5\JQData\KlineH5\options\SQ'
        export_dir = None
        # source_dir = r'\\nas\share\HDF5\JQData\Daily'
        # export_dir = r'\\nas\share\HDF5\JQData\KlineH5\options\SQ'
        list_underlying_name = ['FUTURE']
        # 先生成5m/15m，1分钟看需求
        # list_freqs = ['5m', '15m']
        list_freqs = ['5m']
        skip_check = True
        for underlying_name in list_underlying_name:
            for freqs in list_freqs:
                convert_jq2zq_daily_data(security_type=security_type, start_date=start, end_date=end, freqs=freqs,
                                         underlying_name=underlying_name, fields=fields, source_dir=source_dir,
                                         export_dir=export_dir, skip_check=skip_check)
        pass

    def test_convert_stock_zq_data(self):
        """
        Task:Stock Step2: 更新JQData成分股的行情数据
        :return:
        """
        start = '2021-08-24'
        end = '2021-11-06'
        security_type = 'stock'
        # fields = ['open', 'close', 'high', 'low', 'volume', 'money', 'factor', 'avg']
        fields = ['open', 'close', 'high', 'low', 'volume', 'money', 'factor', 'avg', 'pre_close', 'high_limit',
                  'low_limit']
        freqs = ['1m', '1d']
        convert_jq2zq_daily_data(security_type=security_type, start_date=start, end_date=end, freqs=freqs,
                                 fields=fields,
                                 skip_check=True)
        pass

    def test_convert_index_zq_data(self):
        """
        Task:Index Step2: 更新JQData股票指数的行情数据
        :return:
        """
        start = '2021-07-01'
        end = '2021-07-15'
        security_type = 'index'
        fields = ['open', 'close', 'high', 'low', 'volume', 'money', 'factor', 'avg', 'pre_close', 'high_limit',
                  'low_limit']
        freqs = ['1m', '1d']
        convert_jq2zq_daily_data(security_type=security_type, start_date=start, end_date=end, freqs=freqs,
                                 fields=fields)
        pass

    def test_convert_futures_zq_data(self):
        start = '2010-01-01'
        end = '2017-01-01'
        security_type = 'futures'
        fields = ['open', 'close', 'high', 'low', 'volume', 'money', 'factor', 'avg', 'pre_close', 'high_limit',
                  'low_limit', 'open_interest']
        freqs = ['1m', '1d']
        convert_jq2zq_daily_data(security_type=security_type, start_date=start, end_date=end, freqs=freqs,
                                 fields=fields)

    def test_convert_plane_data(self):
        start = '2004-01-01'
        end = '2020-12-31'
        security_type = 'sw'
        freqs = ['1d']
        convert_jq2zq_daily_data(security_type=security_type, start_date=start, end_date=end, freqs=freqs)

    def test_get_tday_by_interval(self):
        start = '2010-01-01'
        end = '2020-01-31'
        tday_list = _get_trading_days(start_date=start, end_date=end)
        divided_tdays_list = _get_tdays_by_interval(tday_list)
        pass

    def test_convert_stockcode_jq2zq_data(self):
        start = '2019-12-31'
        end = '2019-12-31'
        stockfullcode = ['SH601318']
        freqs = ['1d']
        export_dir = r'D:\TempCSV'
        convert_stockcode_jq2zq_data(start_date=start, end_date=end, stockfullcode=stockfullcode, freqs=freqs,
                                     export_dir=export_dir, replace=True)
        pass

    def test_replace_stock_cache(self):
        start = '2019-12-31'
        end = '2019-12-31'
        freqs = ['1d']
        update_files = list_files(r'\\nas\share\HDF5\JQData\KlineH5\stock', '.h5')
        for each_file in update_files:
            file_name = os.path.basename(each_file)
            stockfullcode = os.path.splitext(file_name)[0]
            stockfullcode = stockfullcode.split('_')[0]
            convert_stockcode_jq2zq_data(start_date=start, end_date=end, stockfullcode=stockfullcode, freqs=freqs,
                                         replace=True)
        pass

    def test_calculate_future_factor_price(self):
        path = r'D:\TempCSV\index_future_cache.h5'
        cache_h5 = pd.read_hdf(path, 'data')
        cache_h5 = cache_h5[cache_h5['contract'] == 'IC9999.CCFX']
        bulk_convert_to_zq_h5(cache_h5, security_type='futures')
        pass

    def test_extract_ed_info_data(self):
        stockfullcode = 'SFIF9001'
        start = '2010-01-01'
        end = '2020-12-31'
        path = r'D:\TempCSV\SFIF9001_kline.h5'
        extract_ed_info_data(stockfullcode=stockfullcode, start=start, end=end, export_path=path)
        pass

    def test_calculate_ed_info_by_oi(self):
        main_stockfullcode_list = ['SFIF9001', 'SFIC9001', 'SFIH9001', 'DLi9001', 'DLj9001', 'SQrb9001', 'SQru9001',
                                   'ZZAP9001', 'SQni9001', 'DLp9001', 'SQfu9001', 'ZZSR9001', 'ZZTA9001', 'ZZMA9001']
        start_date = '2010-01-01'
        end_date = '2020-04-10'
        for main_stockfullcode in main_stockfullcode_list:
            export_path = r'D:\TempCSV\future_data\ed_info\{}_oi_ed_info_list.csv'.format(main_stockfullcode)
            ed_info = calculate_ed_info_by_oi(stockfullcode=main_stockfullcode, export_path=export_path,
                                              start_date=start_date,
                                              end_date=end_date)
            ed_info.to_csv(export_path)
        pass

    def test_extract_new_ed_info_by_oi(self):
        cache_h5_files = list_files(r'D:\TempCSV\future_data\new_ed_info\5m', '.h5')
        for each_file in cache_h5_files:
            file_name = os.path.basename(each_file)
            stockfullcode = os.path.splitext(file_name)[0]
            cache_new_ed_info = pd.read_hdf(each_file, 'edinfo')
            export_path = r'D:\TempCSV\future_data\ed_info\new_ed_info\{}_oi_ed_info_list.csv'.format(stockfullcode)
            cache_new_ed_info.to_csv(export_path)

    def test_cache_future_main_ed_info_data(self):
        main_stockfullcode_list = ['SFIF9001', 'SFIC9001', 'SFIH9001']
        export_path = r'D:\TempCSV\future_data\new_ed_info'
        for main_stockfullcode in main_stockfullcode_list:
            cache_future_main_ed_info_data(stockfullcode=main_stockfullcode, export_dir=export_path, cache_path='http')

    def test_cache_future_main_quote_df(self):
        main_stockfullcode_list = ['SFIF9001', 'SFIC9001', 'SFIH9001', 'DLi9001', 'DLj9001', 'SQrb9001', 'SQru9001',
                                   'ZZAP9001',
                                   'SQni9001', 'DLp9001', 'SQfu9001', 'ZZSR9001', 'ZZTA9001', 'ZZMA9001']
        start_date = '2015-01-01'
        end_date = '2020-03-31'
        path1 = r'D:\TempCSV\future_data\new_ed_info\1m'
        path5 = r'D:\TempCSV\future_data\new_ed_info\5m'
        for main_stockfullcode in main_stockfullcode_list:
            quote_df = get_history_data(stockfullcode=main_stockfullcode, start_date=start_date, end_date=end_date,
                                        freq='5m', ed_type=None, cache_path=path1)
            # quote_df = _get_contract(kline_df=quote_df, jq_stockcode=normalize_code_to_jq(main_stockfullcode),
            #                          security_type='futures')
            quote_df = quote_df[ZQ_KLINE_FORMAT_COLUMNS]

            quote_df.to_hdf(os.path.join(path5, '{}_kline.h5'.format(main_stockfullcode)), '5m', complevel=9,
                            complib='blosc', format='table',
                            data_columns=['date', 'today', 'sts', 'ets'])


if __name__ == "__main__":
    unittest.main()
