# -*- coding:utf-8 -*-
import logging
import os
import datetime as dt
import unittest
import types
import collections


def enum(*sequential, **named):
    enums = dict(zip(sequential, range(len(sequential))), **named)
    reverse = dict((value, key) for key, value in enums.iteritems())
    enums['reverse_mapping'] = reverse
    return type('Enum', (), enums)


class bidict(object):
    def __init__(self, d=None):
        self.pos_dict = {}
        self.neg_dict = {}
        if d is not None:
            self.update(d)

    def update(self, d):
        if not isinstance(d, dict):
            raise TypeError(u'update_bidict不能接受{}类型的参数'.format(type(d)))
        for key, value in d.items():
            self.pos_dict[key] = value
            self.neg_dict[value] = key

    def __getitem__(self, key):
        if key in self.pos_dict:
            return self.pos_dict[key]
        elif key in self.neg_dict:
            return self.neg_dict[key]
        raise KeyError(u'不存在Key为{}的数据'.format(key))

    def __setitem__(self, key, value):
        self.pos_dict[key] = value
        self.neg_dict[value] = key

    def __delitem__(self, key):
        if key in self.pos_dict:
            value = self.pos_dict.pop(key)
            self.neg_dict.pop(value)
            return value
        if key in self.neg_dict:
            value = self.neg_dict.pop(key)
            self.pos_dict.pop(value)
            return value
        return None

    def __contains__(self, item):
        return item in self.pos_dict or item in self.neg_dict

    def __len__(self):
        return len(self.pos_dict)

    def get(self, key, default=None):
        if key in self.pos_dict:
            return self.pos_dict[key]
        elif key in self.neg_dict:
            return self.neg_dict[key]
        else:
            return default

    def clear(self):
        self.pos_dict = {}
        self.neg_dict = {}

    def has_key(self, key):
        return key in self.pos_dict or key in self.neg_dict

    def keys(self):
        return self.pos_dict.keys()

    def values(self):
        return self.pos_dict.values()

    def items(self):
        return self.pos_dict.items()


def group_by(l, key):
    return_dict = collections.defaultdict(list)
    for i in l:
        tmp_key = key(i)
        return_dict[tmp_key].append(i)
    return return_dict


def group_sum_by(l, group_key, sum_key):
    sum_dict = collections.defaultdict(float)
    for i in l:
        tmp_key = group_key(i)
        sum_dict[tmp_key] += sum_key(i)
    return sum_dict


def sum_by(l, key):
    sum = 0
    for i in l:
        sum += key(i)
    return sum


def group_max_by(l, group_key, max_key):
    max_dict = {}
    for i in l:
        tmp_key = group_key(i)
        new_value = max_key(i)
        if not tmp_key in max_dict:
            max_dict[tmp_key] = new_value
        elif new_value > max_dict[tmp_key]:
            max_dict[tmp_key] = new_value
    return max_dict


def max_by(l, key):
    if len(l) == 0:
        return None
    max = key(l[0])
    for i in l:
        if key(i) > max:
            max = key(i)
    return max


def group_min_by(l, group_key, min_key):
    min_dict = {}
    for i in l:
        tmp_key = group_key(i)
        new_value = min_key(i)
        if not tmp_key in min_dict:
            min_dict[tmp_key] = new_value
        elif new_value > min_dict[tmp_key]:
            min_dict[tmp_key] = new_value
    return min_dict


def min_by(l, key):
    if len(l) == 0:
        return None
    min = key(l[0])
    for i in l:
        if key(i) > min:
            min = key(i)
    return min


def inter(a, b):
    """
    判断两个list是否有交集
    :param a:
    :param b:
    :return:
    """
    return list(set(a).intersection(set(b)))

class mytest(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_something(self):
        self.assertEqual(True, False)


def setup_logger(logger_dir='.', name="root", ch_level=logging.INFO, file_level=logging.DEBUG):
    if not os.path.exists(logger_dir):
        os.mkdir(logger_dir)
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)

    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

    ch = logging.StreamHandler()
    ch.setLevel(ch_level)
    ch.setFormatter(formatter)

    log_file = logger_dir + os.sep + os.path.basename(os.path.abspath(__file__)) + ".log"

    fh = logging.FileHandler(log_file, "w")
    fh.setLevel(file_level)
    fh.setFormatter(formatter)

    logger.addHandler(fh)
    logger.addHandler(ch)

    return logger, fh


if __name__ == "__main__":
    logger_dir = os.path.dirname(os.path.abspath(__file__)) + os.sep + "log" + dt.datetime.today().strftime("%m%d")
    logger, file_handler = setup_logger(logger_dir=logger_dir, name="root", ch_level=logging.INFO,
                                        file_level=logging.DEBUG)
    unittest.main()
