# -*- conding:UTF-8 -*-
import json
import unittest
import requests
import traceback
import os
import logging
import configparser
from retrying import retry
from rio3.log.setup import setup_file_screen_logger

logger = logging.getLogger(__name__)
setup_file_screen_logger(name=__name__)


class BaseRequests(object):

    @retry(stop_max_attempt_number=10, wait_fixed=5000)
    def get(self, url, params=None, headers=None, files=None, auth=None):
        """
        get请求，返回响应码和响应实体
        :param url:
        :param params:
        :param headers:
        :param files:
        :param auth:
        :return:
        """
        try:
            r = requests.get(url, params=params, headers=headers, files=files, auth=auth, verify=False, stream=True,
                             timeout=10)
            logger.info(msg='请求实体内容：%s' % params)
            status_code = r.status_code
            logger.info(msg='返回状态码：%d' % status_code)
            response = r.json()
            logger.info(msg='响应实体内容：%s' % response)
            return status_code, response
        except BaseException as ex:
            self.log_request_exception(ex=ex, url=url, params=params)

    @retry(stop_max_attempt_number=10, wait_fixed=5000)
    def post(self, url, data=None, headers=None, files=None, auth=None, timeout=10):
        """
        post请求，返回响应码和响应实体
        :param url:
        :param data:
        :param headers:
        :param files:
        :param auth:
        :param timeout:
        :return:
        """
        status_code = -1
        response = list()
        try:
            r = requests.post(url, data=data, headers=headers, files=files, auth=auth, verify=False, timeout=timeout)
            logger.info(msg='请求实体内容：%s' % data)
            status_code = r.status_code
            logger.info(msg='返回状态码：%d' % status_code)
            response = r.json()
            logger.info(msg='响应实体内容：%s' % response)
        except BaseException as ex:
            self.log_request_exception(ex=ex, url=url, params=data)
        finally:
            return status_code, response

    @retry(stop_max_attempt_number=10, wait_fixed=5000)
    def post_json(self, url, data=None, headers=None, auth=None):
        """post请求，使用json格式提交请求实体，返回响应码和响应实体"""
        try:
            r = requests.post(url, json=data, headers=headers, auth=auth, verify=False, timeout=10)
            logger.info(msg='请求实体内容：%s' % data)
            status_code = r.status_code
            logger.info(msg='返回状态码：%d' % status_code)
            response = r.json()
            logger.info(msg='响应实体内容：%s' % response)
            return status_code, response
        except BaseException as ex:
            self.log_request_exception(ex=ex, url=url, params=data)

    def dict_value(self, dict1, obj, default=None):
        """查找嵌套字典中key对应的值"""
        try:
            for k, v in dict1.items():
                if k == obj:
                    print('嵌套字典中%s对应的值为%s' % (obj, v))
                    return v
                else:
                    if type(v) is dict:
                        re = self.dict_value(v, obj, default)
                        if re is not default:
                            return re
        except BaseException:
            print('可能有非字典嵌套', exc_info=1)

    def get_addkey(self, user):
        """遍历获得配置文件收件人email"""
        sum1 = 0
        L = []
        for i in user:
            if sum1 < len(user):
                emails = self.config_read('addressed', i)
                L.append(emails)
                sum1 += 1
        return L

    @staticmethod
    def log_request_exception(ex=None, url=None, params=None):
        logger.error(f'[请求失败]\n[url]:{url}\n[params]:{params}\n[error]:{ex}\n[traceback]:{traceback.format_exc()}\n')
